#!/usr/bin/env python
from __future__ import division

import rlg2mpl
import matplotlib.colors
import matplotlib.ticker
import matplotlib.transforms
from matplotlib.text import Text
from matplotlib.patches import PathPatch
from matplotlib.font_manager import FontProperties
from matplotlib.collections import (CircleCollection, PolyCollection, 
    LineCollection, RegularPolyCollection)
from matplotlib.transforms import (IdentityTransform, 
    blended_transform_factory, Affine2DBase)
import numpy

import copy
import warnings
from cogent.core.location import Map, Span, _norm_slice
from cogent.core.moltype import DNA

__author__ = "Peter Maxwell"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Gavin Huttley", "Peter Maxwell", "Matthew Wakefield"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Peter Maxwell"
__email__ = "pm67nz@gmail.com"
__status__ = "Production"

# map - a 1D space.  must support len() and in some cases [i]
# track - a panel within a sequence display holding annotations
# display - a collection of stacked tracks with a shared base sequence
# base - the map that provides the X scale
# annotation - an annotated disjoint location on a map
# span - a contiguous part of an annotation
# feature, code, variable, shade - types of anotation

# TODO: subtracks, spectra/width variables, strides, windows
#       broken ends, vscales, zero lines, sub-pixel features.
#       fuzzy ends.  more ids/qualifiers.

MAX_WIDTH = 72*8

def dna_shapes():
    w = .5 / 2
    s = .75 / 2
    m = (w+s) / 2
    r = 8
    y = 5
    n = (r+y) / 2
    def rectangle(x,y):
        return [(-x, 0), (-x, y), (x, y), (x, 0)]
    shapes = {}
    for (motif, width, height) in [
            ('A', w, r),
            ('G', s, r),
            ('C', s, y),
            ('T', w, y),
            ('R', m, r),
            ('Y', m, y),
            ('S', s, n),
            ('W', w, n),
            ('N', m, n)]:
        shapes[motif] = rectangle(width, height)
    return shapes
    
dna_shapes = dna_shapes()

class TransformScalePart(Affine2DBase):
    """Just the translation factors of the child transform, no
    rotation or translation. 
    
    a: Child transform from which scale is extracted
    source_dims: the dimensions (0:X, 1:Y, 2:I) from which the
    resulting X and Y scales are taken."""
    
    def __init__(self, a, source_dims=[0,1]):
        self.input_dims = a.input_dims
        self.output_dims = a.output_dims
        self._a = a
        self._scale_source_dims = source_dims
        self._mtx = a.get_affine().get_matrix().copy()
        self._mtx[:] = 0
        self._mtx[2,2] = 1
        self._invalid = True
        self.set_children(a)
        Affine2DBase.__init__(self)

    def get_matrix(self):
        if self._invalid:
            a = self._a.get_affine().get_matrix()
            for dim in [0,1]:
                sdim = self._scale_source_dims[dim]
                self._mtx[dim,dim] = a[sdim,sdim]
            self._inverted = None
            self._invalid = 0
        return self._mtx

        
class _Colors(object):
    """colors.white = to_rgb("white"), same as just using "white"
    except that this lookup also checks the color is valid"""   
    def __getattr__(self, attr):
        return matplotlib.colors.colorConverter.to_rgb(attr)
colors = _Colors()

def llen(label, fontSize=10):
    # placeholder for better length-of-label code
    return len(label) * fontSize
    
class TrackDefn(object):
    def __init__(self, tag, features):
        assert tag
        self.tag = tag
        self.feature_types = features
    
    def __iter__(self):
        return iter(self.feature_types)
    

class Track(object):
    def __init__(self, tag, features, level=0, label=None,
            needs_border=False, max_y=None, height=None):
        assert tag
        if label is None:
            label = tag
        self.tag = tag
        self.label = label
        assert isinstance(label, str), (tag, label)
        self.features = features
        self.height = max(height, max([f.height for f in self.features]))
        self.range = max_y or max(
            # xxx this works for zero-based only
            [getattr(f, 'value', None) for f in self.features]) or 0
        self.level = level
        self.needs_border = needs_border
    
    def getShapes(self, span, rotated, height, 
            yrange=None, done_border=False):
        shape_list = [feature.shape(height,
                yrange or self.range, rotated) for feature in self.features]
        if self.needs_border and not done_border:
            border = rlg2mpl.Group(
                    rlg2mpl.Line(span[0], 0, span[1], 0,
                            strokeWidth=.5, strokeColor=colors.black),
                    rlg2mpl.Line(span[0], height, span[1], height,
                            strokeWidth=.5, strokeColor=colors.black)
                    )
            shape_list = [border] + shape_list
        return shape_list
    
    def __repr__(self):
        return "Track(%(tag)s,%(label)s)" % vars(self)
    

class CompositeTrack(Track):
    """Overlayed tracks"""
    def __init__(self, tag, tracks, label=None):
        if label is None:
            labels = dict([(track.label, True)
                    for track in tracks]).keys()
            if len(labels) == 1:
                label = labels[0]
            else:
                label = ''
        self.tag = tag
        self.label = label
        self.tracks = tracks
        self.height = max([track.height for track in tracks])
        self.level = max([track.level for track in tracks])
        self.range = max([track.range for track in tracks])
    
    def getShapes(self, span, rotated, height, 
            yrange=None, done_border=False):
        if yrange is None:
            yrange = self.range
        shape_list = []
        for track in self.tracks:
            if track.needs_border and not done_border:
                border = rlg2mpl.Group(
                        rlg2mpl.Line(span[0], 0, span[1], 0,
                                strokeWidth=.5, strokeColor=colors.black),
                        rlg2mpl.Line(span[0], height, span[1], height,
                                strokeWidth=.5, strokeColor=colors.black)
                        )
                shape_list.append(border)
                done_border = True
            shape_list.extend(track.getShapes(span, rotated, height,
                    yrange=yrange, done_border=True))
        return shape_list
    

class Annotation(object):
    """A map, a style, and some values"""
    
    def __init__(self, map, *args, **kw):
        self.map = map
        self.values = self._make_values(*args, **kw)
    
    def _make_values(self, *args, **kw):
        # override for variables etc.
        return []
    
    def __repr__(self):
        return "%s at %s" % (type(self), getattr(self, 'map', '?'))
    
    # xxx styles do this.  what about maps/ others
    def shape(self, height, yrange, rotated):
        g = rlg2mpl.Group()
        posn = 0
        for span in self.map.spans:
            if not span.lost:
                g.add(self._item_shape(
                        span, self.values[posn:posn+span.length],
                        height, yrange, rotated))
            posn += span.length
        return g
    
class _SeqRepresentation(object):
    height = 20
    y_offset = 10
    x_offset = 0

    def __init__(self, map, sequence, cvalues=None, colour_sequences=True, 
            font_properties=None):
        self.font_properties = font_properties
        
        alphabet = self.alphabet = sequence.MolType.Alphabets.Degen
        
        alphabet_colours = None 
        if cvalues:
            assert len(cvalues) == len(sequence)
            cvalues = numpy.asarray(cvalues)
        elif colour_sequences:
            colour_map = sequence.getColourScheme(colors)
            color_specs = [colour_map.get(m,'grey') for m in self.alphabet]
            alphabet_colours = numpy.array([
                matplotlib.colors.colorConverter.to_rgba(c, alpha=.5)
                for c in color_specs])
        
        # this could be faster is sequence were known to be a ModelSequence
        sequence = numpy.asarray(self.alphabet.toIndices(str(sequence)))
        
        posn = 0
        used_count = 0
        offsets = []
        lengths = []
        used = numpy.zeros([len(sequence)], bool)
        x_offset = self.x_offset * 1.0
        for span in map.spans:
            if not (span.lost or span.Reverse):
                offsets.append(x_offset + span.Start - used_count)
                lengths.append(span.length)
                used[posn:posn+span.length] = True
                used_count += span.length
            posn += span.length
        seq = sequence[used]
        if cvalues is None:
            cvals = None
        else:
            cvals = cvalues[used]
        x_offsets = numpy.repeat(offsets, lengths) + numpy.arange(used_count)
        y_offsets = numpy.zeros_like(x_offsets) + self.y_offset
        offsets = numpy.vstack([x_offsets, y_offsets]).T
        self._calc_values(seq, cvals, alphabet_colours, offsets)

    def shape(self, height, yrange, rotated):
        raise NotImplementedError
        

class _MultiShapeSeqRepresentation(_SeqRepresentation):
    def _calc_values(self, sequence, cvalues, alphabet_colours, offsets):
        motifs = range(len(self.alphabet))
        values = []
        for motif in motifs:
            positions = numpy.flatnonzero(sequence==motif)
            if len(positions) == 0:
                continue
            if cvalues is not None:
                cvs = cvalues.take(positions, axis=0)
            elif alphabet_colours is not None:
                cvs = [alphabet_colours[motif]]
            else:
                cvs = [colors.black]
            values.append((motif, cvs, offsets[positions]))
        self.per_shape_values = values
            

class _SingleShapeSeqRepresentation(_SeqRepresentation):
    def _calc_values(self, sequence, cvalues, alphabet_colours, offsets):
        if cvalues:
            cvs = cvalues
        elif alphabet_colours is not None:
            cvs = alphabet_colours.take(sequence, axis=0)
        else:
            cvs = [colors.black] 
        self.cvalues = cvs
        self.offsets = offsets


class SeqText(_MultiShapeSeqRepresentation):
    height = 20
    x_offset = 0.5

    def shape(self, height, yrange, rotated):
        rot = 0
        if rotated: rot += 90
        #if span.Reverse: rot+= 180
        g = rlg2mpl.Group()
        kw = dict(ha='center', va='baseline', rotation=rot,
                font_properties=self.font_properties)
        for (motif, cvalues, offsets) in self.per_shape_values:
            letter = self.alphabet[motif]
            c = len(cvalues)
            for (i, (x,y)) in enumerate(offsets):
                s = Text(x, y, letter, color=cvalues[i%c], **kw)
                g.add(s)
        return g

class SeqShapes(_MultiShapeSeqRepresentation):
    height = 10
    x_offset = 0.5
    y_offset = 0
    
    def __init__(self, map, sequence, *args, **kw):
        super(SeqShapes, self).__init__(map, sequence, *args, **kw)
        default = dna_shapes['N']
        self.shapes = [dna_shapes.get(m, default) for m in self.alphabet]
        self.rshapes = [[(y,x) for (x,y) in v] for v in self.shapes]
        
    def shape(self, height, yrange, rotated):
        g = rlg2mpl.Group()
        (X, Y, I) = (0, 1, 2)
        shapes = [self.shapes, self.rshapes][rotated]
        trans = TransformScalePart(g.combined_transform)
        artists = []
        for (motif, cvalues, offsets) in self.per_shape_values:
            shape = shapes[motif]
            a = PolyCollection([shape], closed=True,
                facecolors=cvalues, edgecolors=cvalues, offsets=offsets, 
                transOffset=g.combined_transform)
            g.add(a)
            a.set_transform(trans)
        return g

class SeqDots(_SingleShapeSeqRepresentation):
    # Something wrong with this one.
    height = 5
    x_offset = 0.5
    y_offset = 5

    def shape(self, height, yrange, rotated):
        g = rlg2mpl.Group()
        (X, Y, I) = (0, 1, 2)
        #scaled_axes = [[X, I], [I, Y]][rotated]
        scaled_axes = [[X, X], [Y, Y]][rotated]
        scaled_axes = [[X, Y], [X, Y]][rotated]
        trans = TransformScalePart(g.combined_transform, scaled_axes)
        a = CircleCollection([.5], edgecolors=self.cvalues,
                facecolors=self.cvalues, offsets=self.offsets,
                transOffset=g.combined_transform)
        g.add(a)
        a.set_transform(trans)
        return g

class SeqLineSegments(_SingleShapeSeqRepresentation):
    height = 5
    x_offset = 0.0
    y_offset = 2.5

    def shape(self, height, yrange, rotated):
        g = rlg2mpl.Group()
        trans = TransformScalePart(g.combined_transform)
        segment = [(.1,0),(.9,0)]
        if rotated:
            segment = [(y,x) for (x,y) in segment]
        a = LineCollection([segment], colors=self.cvalues, 
                offsets=self.offsets, transOffset=g.combined_transform)
        a.set_linewidth(3)
        g.add(a)
        a.set_transform(trans)
        return g


class SeqLine(object):
    height = 20

    def __init__(self, map, *args, **kw):
        x_offset = 0.0
        self.segments = [(span.Start+x_offset, span.End+x_offset)
            for span in map.spans if not span.lost]
            
    def shape(self, height, yrange, rotated):
        g = rlg2mpl.Group()
        trans = TransformScalePart(g.combined_transform)
        y = height/2.0
        segments = [[(x1,y),(x2,y)] for (x1,x2) in self.segments]
        a = LineCollection(segments, edgecolor='k', facecolor='k')
        a.set_linewidth(2)
        g.add(a)
        a.set_transform(g.combined_transform)
        return g


class Feature(Annotation):
    """An Annotation with a style and location rather than values"""
    def __init__(self, map, style, label=None, value=None):
        self.map = map
        self.style = style
        self.label = label
        self.value = value
        self.height = style.height
        #self.values = self._make_values(*args, **kw)
    
    def shape(self, height, yrange, rotated):
        return self.style(height, self.label, self.map, self.value, yrange, 
                rotated)
    

class _FeatureStyle(object):
    range_required = False
    def __init__(self, fill=True, color=colors.black, min_width=0.5,
            showLabel=False, height=1, thickness=0.6, closed=True,
            one_span=False, **kw):
        opts = {}
        if fill:
            opts['fillColor'] = color
            opts['strokeColor'] = None
            opts['strokeWidth'] = 0
        else:
            opts['fillColor'] = colors.white # otherwise matplotlib blue!
            opts['strokeColor'] = color
            opts['strokeWidth'] = 1
        self.filled = fill
        self.closed = closed or fill
        opts.update(kw)
        self.opts = opts
        self.min_width = min_width
        self.showLabel = showLabel
        self.height = height
        self.proportion_of_track = thickness
        self.one_span = one_span
    
    def __call__(self, height, label, map, value, yrange, rotated):
        #return self.FeatureClass(label, map)
        g = rlg2mpl.Group()
        last = first = None
        if self.range_required and not yrange:
            warnings.warn("'%s' graph values are all zero" % label)
            yrange = 1.0
        if map.useful and self.one_span:
            map = map.getCoveringSpan()
        for (i, span) in enumerate(map.spans):
            #if last is not None:
            #    g.add(rlg2mpl.Line(last, height, part.Start, height))
            if span.lost or (value is None and self.range_required):
                continue
            if span.Reverse:
                (start, end) = (span.End, span.Start)
                (tidy_start, tidy_end) = (span.tidy_end, span.tidy_start)
            else:
                (start, end) = (span.Start, span.End)
                (tidy_start, tidy_end) = (span.tidy_start, span.tidy_end)
            shape = self._item_shape(
                start, end,
                tidy_start, tidy_end, height, value, yrange, rotated, 
                last=i==len(map.spans)-1)
            g.add(shape)
            last = end
            if first is None:
                first = start
        if self.showLabel and label and last is not None and height > 7:
            font_height = 12 #self.label_font.get_size_in_points()
            text_width = llen(label, font_height)
            if (text_width < abs(first-last)):
                label_shape = Text(
                    (first+last)/2, height/2, label,
                    ha="center", va="center", 
                    rotation=[0,90][rotated],
                    #font_properties=self.label_font,
                    )
                g.add(label_shape)
            else:
                pass #warnings.warn("couldn't fit feature label '%s'" % label)
        return g

class _VariableThicknessFeatureStyle(_FeatureStyle):
    def _item_shape(self, start, end, tidy_start, tidy_end, height, value,
            yrange, rotated, last=False):
        if yrange:
            thickness = 1.0*value/yrange*height
        else:
            thickness = height*self.proportion_of_track
        return self._item_shape_scaled(start, end, tidy_start, tidy_end,
                height/2, max(2, thickness), rotated, last)
    
class Box(_VariableThicknessFeatureStyle):
    arrow = False
    blunt = False
    
    def _item_shape_scaled(self, start, end, tidy_start, tidy_end, middle,
            thickness, rotated, last):
        (top, bottom) = (middle+thickness/2, middle-thickness/2)
        kw = dict(min_width=self.min_width, pointy=False, closed=self.closed, 
            blunt=self.blunt, proportion_of_track=self.proportion_of_track)
        kw['rounded'] = tidy_start
        #kw['closed'] = self.closed or tidy_start
        end1 = rlg2mpl.End(start, end, bottom, top, **kw)
        kw['rounded'] = tidy_end
        #kw['closed'] = self.closed or tidy_end or self.filled
        kw['pointy'] = last and self.arrow
        end2 = rlg2mpl.End(end, start, top, bottom, **kw)
        path = end1 + end2
        return PathPatch(path, **rlg2mpl.line_options(**self.opts))   

class Arrow(Box):
    arrow = True
    blunt = False

class BluntArrow(Box):
    arrow = True
    blunt = True

class Diamond(_VariableThicknessFeatureStyle):
    """diamond"""
    def _item_shape_scaled(self, start, end, tidy_start, tidy_end, middle,
            thickness, rotated, last):
        x = (start+end)/2
        spread = max(abs(start-end), self.min_width) / 2
        return rlg2mpl.Polygon(
            [(x-spread, middle), (x, middle+thickness/2), (x+spread, middle), 
            (x, middle-thickness/2)], **self.opts)
    

class Line(_FeatureStyle):
    """For a line segment graph"""
    range_required = True
    def _item_shape(self, start, end, tidy_start, tidy_end, height, value,
            yrange, rotated, last=False):
        altitude = value * (height-1) / yrange
        #if self.orientation < 0:
        #    altitude = height - altitude
        return rlg2mpl.Line(start, altitude, end, altitude, **self.opts)
    

class Area(_FeatureStyle):
    """For a line segment graph"""
    range_required = True
    def _item_shape(self, start, end, tidy_start, tidy_end, height, value,
            yrange, rotated, last=False):
        altitude = value * (height-1) / yrange
        #if self.orientation < 0:
        #    altitude = height - altitude
        if end < start:
            start, end = end, start
            tidy_start, tidy_end = tidy_end, tidy_start
        return rlg2mpl.Rect(start, 0, end-start, altitude, **self.opts)
    

class DisplayPolicy(object):
    def _makeFeatureStyles(self):
        return {
            #gene structure
            'misc_RNA': Box(True, colors.lightcyan),
            'precursor_RNA': Box(True, colors.lightcyan),
            'prim_transcript': Box(True, colors.lightcyan),
            "3'clip": Box(True, colors.lightcyan),
            "5'clip": Box(True, colors.lightcyan),
            'mRNA': Box(True, colors.cyan),
            'exon': Box(True, colors.cyan),
            'intron': Box(False, colors.cyan, closed = False),
            "3'UTR": Box(True, colors.cyan),
            "5'UTR": Box(True, colors.cyan),
            'CDS': Box(True, colors.blue),
            'mat_peptide': Box(True, colors.blue),
            'sig_peptide': Box(True, colors.navy),
            'transit_peptide': Box(True, colors.navy),
            'polyA_signal': Box(True, colors.lightgreen),
            'polyA_site': Diamond(True, colors.lightgreen),
            'gene': BluntArrow(False, colors.blue,
                    showLabel=True, closed = False),
            'operon': BluntArrow(False, colors.royalblue,
                    showLabel=True, closed = False),
            #regulation
            'attenuator': Box(False, colors.red),
            'enhancer': Box(True, colors.green),
            'CAAT_signal': Diamond(True, colors.blue),
            'TATA_signal': Diamond(True, colors.teal),
            'promoter': Box(False, colors.seagreen),
            'GC_signal': Box(True, colors.purple),
            'protein_bind': Box(True, colors.orange),
            'misc_binding': Box(False, colors.black),
            '-10_signal': Diamond(True, colors.blue),
            '-35_signal': Diamond(True, colors.teal),
            'terminator': Diamond(True, colors.red),
            'misc_signal': Box(False, colors.maroon),
            'rep_origin': Box(True, colors.linen),
            'RBS': Diamond(True, colors.navy),
            #repeats
            'repeat_region': Box(True, colors.brown),
            'repeat_unit': Arrow(True, colors.brown),
            'LTR': Box(False, colors.black),
            'satellite': Box(False, colors.brown),
            'stem_loop': Box(False, colors.dimgray),
            'misc_structure': Box(False, colors.darkslategray),
            #rna genes
            'rRNA': Arrow(False, colors.darkorchid, showLabel=True),
            'scRNA': Arrow(False, colors.darkslateblue, showLabel=True),
            'snRNA': Arrow(False, colors.darkviolet, showLabel=True),
            'snoRNA': Arrow(False, colors.darkviolet, showLabel=True),
            'tRNA': Arrow(False, colors.darkturquoise, showLabel=True),
            #sequence
            'source': Box(False, colors.black, showLabel=True),
            'misc_recomb': Box(False, colors.black, showLabel=True),
            'variation': Diamond(True, colors.violet, showLabel=True),
            'domain': Box(False, colors.darkorange, showLabel=True),
            'bluediamond': Diamond(True, colors.blue),
            'reddiamond': Diamond(True, colors.red),
            'misc_feature': Box(True, colors.darkorange, showLabel=True),
            'old_sequence': Box(False, colors.darkslategray),
            'unsure': Diamond(False, colors.crimson, min_width=2,),
            'misc_difference': Diamond(False, colors.darkorange),
            'conflict': Box(False, colors.darkorange),
            'modified_base': Diamond(True, colors.black),
            'primer_bind': Arrow(False, colors.green, showLabel=True),
            'STS': Box(False, colors.black),
            'gap': Box(True, colors.gray),
            #graphs
            'blueline': Line(False, colors.blue),
            'redline': Line(False, colors.red),
            #other
            ##immune system specific
            #'C_region': Diamond(True, colors.mediumblue),
            #'N_region': Box(False, colors.linen),
            #'S_region': Box(False, colors.linen),
            #'V_region': Box(False, colors.linen),
            #'D_segment': Diamond(True, colors.mediumpurple),
            #'J_segment': Box(False, colors.linen),
            #'V_segment': Box(False, colors.linen),
            #'iDNA': Box(False, colors.grey),
            ##Mitocondria specific
            #'D-loop': Diamond(True, colors.linen),
            ##Bacterial element specific
            #'oriT': Box(False, colors.linen),
        }
    
    def _makeTrackDefns(self):
        return [TrackDefn(*args) for args in [
            ('Gene Structure',[
                'misc_RNA',
                'precursor_RNA',
                'prim_transcript',
                "3'clip",
                "5'clip",
                'mRNA',
                'exon',
                'intron',
                "3'UTR",
                "5'UTR",
                'CDS',
                'mat_peptide',
                'sig_peptide',
                'transit_peptide',
                'polyA_signal',
                'polyA_site',
                'gene',
                'operon',
                ]),
            ('Regulation',[
                'attenuator',
                'enhancer',
                'CAAT_signal',
                'TATA_signal',
                'promoter',
                'GC_signal',
                'protein_bind',
                'misc_binding',
                '-10_signal',
                '-35_signal',
                'terminator',
                'misc_signal',
                'rep_origin',
                'RBS',
                ]),
            ('Repeats',[
                'repeat_region',
                'repeat_unit',
                'LTR',
                'satellite',
                'stem_loop',
                'misc_structure',
                ]),
            ('Rna Genes',[
                'rRNA',
                'scRNA',
                'snRNA',
                'snoRNA',
                'tRNA',
                ]),
            ('Sequence',[
                'source',
                'misc_recomb',
                'domain',
                'variation',
                'bluediamond',
                'reddiamond',
                'misc_feature',
                'old_sequence',
                'unsure',
                'misc_difference',
                'conflict',
                'modified_base',
                'primer_bind',
                'STS',
                'gap',
                ]),
            ('Graphs',[
                'blueline',
                'redline',
                ]),
        ]]
    
    _default_ignored_features =    ['C_region','N_region','S_region','V_region',
        'D_segment','J_segment','V_segment','iDNA','D-loop','oriT',]
    _default_keep_unexpected_tracks = True
    
    dont_merge = []
    show_text = None # auto
    draw_bases = None
    show_gaps = None
    colour_sequences = None
    seq_color_callback = None
    seqname = ''
    rowlen = None
    recursive = True
    
    def __init__(self,
            min_feature_height = 20,
            min_graph_height = None,
            ignored_features=None,
            keep_unexpected_tracks=None,
            **kw):
        
        self.seq_font = FontProperties(size=10)
        #self.label_font = FontProperties()

        if min_graph_height is None:
            min_graph_height = min_feature_height * 2
        
        feature_styles = self._makeFeatureStyles()
        # yuk
        for style in feature_styles.values():
            if style.range_required:
                style.height = max(style.height, min_graph_height)
            else:
                style.height = max(style.height, min_feature_height)
        
        self._track_defns = self._makeTrackDefns()
        
        if ignored_features is None:
            ignored_features = self._default_ignored_features
        self._ignored_features = ignored_features
        if keep_unexpected_tracks is None:
            keep_unexpected_tracks = self._default_keep_unexpected_tracks
        self.keep_unexpected_tracks = keep_unexpected_tracks
        
        if not hasattr(self, '_track_map'):
            self._track_map = {}
            for track_defn in self._track_defns:
                for (level, feature_tag) in enumerate(track_defn):
                    feature_style = feature_styles[feature_tag]
                    self._track_map[feature_tag] = (
                            track_defn, level, feature_style)
            for ft in self._ignored_features:
                self._track_map[ft] = (None, 0, None)
            for ft in feature_styles:
                if ft not in self._track_map:
                    self._track_map[ft] = (None, level, feature_style)
        self.map = None
        self.depth = 0
        self.orientation = -1
        self.show_code = True
        self._logged_drops = []
        
        self._setattrs(**kw)
    
    def _setattrs(self, **kw):                 
        for (n,v) in kw.items():
            if not hasattr(self, n):
                warnings.warn('surprising kwarg "%s"' % n, stacklevel=3)
            if n.endswith('font'):
                assert isinstance(kw[n], FontProperties)
            setattr(self, n, v)
    
    def copy(self, **kw):
        new = copy.copy(self)
        new._setattrs(**kw)
        return new
    
    def at(self, map):
        if map is None:
            return self
        else:
            return self.copy(map=self.map[map], depth=self.depth+1)
    
    def mergeTracks(self, orig_tracks, keep_unexpected=None):
        # merge tracks with same names
        # order features within a track by level  # xxx remerge
        tracks = {}
        orig_track_tags = []
        for track in orig_tracks:
            if not track.tag in tracks:
                tracks[track.tag] = {}
                orig_track_tags.append(track.tag) # ordered list
            if not track.level in tracks[track.tag]:
                tracks[track.tag][track.level] = []
            tracks[track.tag][track.level].append(track)
        
        track_order = [track.tag for track in self._track_defns
                if track.tag in tracks]
        unexpected = [tag for tag in orig_track_tags if tag not in track_order]
        if keep_unexpected is None:
            keep_unexpected = self.keep_unexpected_tracks
        if keep_unexpected:
            track_order += unexpected
        elif unexpected:
            warnings.warn('dropped tracks ' + ','.join(unexpected), stacklevel=2)
        
        sorted_tracks = []
        for track_tag in track_order:
            annots = []
            levels = tracks[track_tag].keys()
            levels.sort()
            for level in levels:
                annots.extend(tracks[track_tag][level])
            if len(annots)> 1 and track_tag not in self.dont_merge:
                sorted_tracks.append(CompositeTrack(track_tag, annots))
            else:
                sorted_tracks.extend(annots)
        return sorted_tracks
    
    def tracksForAlignment(self, alignment):
        annot_tracks = alignment.getAnnotationTracks(self)
        if self.recursive:
            if self.show_gaps is None:
                seqs_policy = self.copy(show_gaps=True)
            else:
                seqs_policy = self
            seq_tracks = alignment.getChildTracks(seqs_policy)
        else:
            seq_tracks = []
        annot_tracks = self.mergeTracks(annot_tracks)
        return seq_tracks + annot_tracks
    
    def tracksForSequence(self, sequence=None):
        result = []
        length = None
        if length is None and sequence is not None:
            length = len(sequence)
        
        label = getattr(self, 'seqname', '')
        
        if self.show_code and sequence is not None:
            # this should be based on resolution, not rowlen, but that's all
            # we have at this point
            if self.seq_color_callback is not None:
                cvalues = self.seq_color_callback(sequence)
            else:
                cvalues = None
            show_text = self.show_text
            draw_bases = self.draw_bases
            if draw_bases is None:
                draw_bases = self.rowlen <= 500 and sequence.MolType is DNA
            self.rowlen <= 500 and sequence.MolType is DNA and self.draw_bases
            if show_text is None:
                show_text = self.rowlen <= 100
            if show_text and self.rowlen <= 200:
                seqrepr_class = SeqText
            elif draw_bases:
                seqrepr_class = SeqShapes
            elif self.rowlen <= 1000 and (self.colour_sequences 
                    or cvalues is not None):
                seqrepr_class = SeqLineSegments
            elif self.show_gaps:
                seqrepr_class = SeqLine
            else:
                seqrepr_class = None
            if seqrepr_class is not None:
                colour_sequences = self.colour_sequences 
                if colour_sequences is None:
                    colour_sequences = seqrepr_class != SeqText
                feature = seqrepr_class(self.map, sequence, 
                        colour_sequences = colour_sequences,
                        font_properties = self.seq_font,
                        cvalues = cvalues)
                result.append(Track('seq', [feature], level=2, label=label))
        else:
            pass
            # show label somewhere
        
        annot_tracks = sequence.getAnnotationTracks(self)
        return self.mergeTracks(annot_tracks + result)
    
    def getStyleDefnForFeature(self, feature):
        if feature.type in self._track_map:
            (track_defn, level, style) = self._track_map[feature.type]
        elif self.keep_unexpected_tracks:
            (track_defn, level, style) = self._track_map['misc_feature']
        else:
            if feature.type not in self._logged_drops:
                warnings.warn('dropped feature ' + repr(feature.type))
                self._logged_drops.append(feature.type)
            return (None, None, None)
        
        if track_defn is None:
            warnings.warn('dropped feature ' + repr(feature.type))
            return (None, None, None)
        else:
            track_tag = track_defn.tag or feature.type
        return (track_tag, style, level)
    
    def tracksForFeature(self, feature):
        (track_tag, style, level) = self.getStyleDefnForFeature(feature)
        if style is None:
            return []
        annot_tracks = feature.getAnnotationTracks(self)
        return annot_tracks + [Track(track_tag,
                [Feature(self.map, style, feature.Name)], level=level)]
    
    def tracksForVariable(self, variable):
        (track_tag, style, level) = self.getStyleDefnForFeature(variable)
        if style is None:
            return []
        segments = []
        max_y = 0.0
        for ((x1, x2), y) in variable.xxy_list:
            map = self.map[x1:x2]
            segments.append(Feature(map, style, variable.Name, value=y))
            if type(y) is tuple: y = max(y)
            if y > max_y: max_y = y
        return [Track(track_tag, segments, max_y=max_y, needs_border=True,
                label=variable.Name, level=level)]
    

class Display(rlg2mpl.Drawable):
    """Holds a list of tracks and displays them all aligned
    
    base: A sequence, alignment, or anything else offering .getTracks(policy)
    policy: A DisplayPolicy subclass.
    pad: Gap between tracks in points.
    
    Other keyword arguments are used to modify the DisplayPolicy: 
    
    Sequence display:
    show_text: Represent bases as characters.  Slow.
    draw_bases: Represent bases as rectangles if MolType allows.
    show_gaps: Represent bases as line segments.
    colour_sequences: Colour code sequences if MolType allows.
    seq_color_callback: f(seq)->[colours] for flexible seq coloring.
    
    Layout:
    rowlen: wrap at this many characters per line.
    min_feature_height: minimum feature symbol height in points.
    min_graph_height: minimum height of any graphed features in points.
    
    Inclusion:
    recursive: include the sequences of the alignment.
    ignored_features: list of feature type tags to leave out.
    keep_unexpected_tracks: show features not assigned to a track by the policy.
    """
    
    def __init__(self, base, policy=DisplayPolicy, _policy=None, pad=1,
            yrange=None, **kw):
        self.pad = pad
        self.base = base
        self.yrange = yrange
        assert len(base) > 0, len(base)
        
        if _policy is None:
            policy = policy(**kw).copy(
                map=Map([(0, len(base))], parent_length=len(base)),
                depth=0, 
                rowlen=len(base))
        else:
            policy = _policy
        self.policy = policy
        self.smap=Map([(0, len(base))], parent_length=len(base))

        self._calc_tracks()
    
    def __len__(self):
        return len(self.smap.inverse())
    
    def _calc_tracks(self):
        y = 0
        self._tracks = []
        for p in self.base.getTracks(self.policy)[::-1]:
            if not isinstance(p, Track):
                if not isinstance(p, list):
                    p = [p]
                p = Track('', p)
            y2 = y + p.height + self.pad
            self._tracks.append((y+self.pad/2, (y+y2)/2, p))
            y = y2
        self.height = y
        
        if self.yrange is None:
            self.yrange = {}
            for (y, ym, p) in self._tracks:
                self.yrange[p.tag] = max(self.yrange.get(p.tag, 0), p.range)
        
    def copy(self, **kw):
        new = copy.copy(self)
        new.policy = self.policy.copy(**kw)
        new._calc_tracks()
        return new
    
    def __getitem__(self, slice):
        c = copy.copy(self)
        c.smap = self.smap.inverse()[slice].inverse()
        return c
        
    def makeArtist(self, vertical=False):
        g = rlg2mpl.Group()
        for (y, ym, p) in self._tracks:
            smap = self.smap.inverse()
            for s in p.getShapes(
                    span=(smap.Start, smap.End),
                    rotated=vertical,
                    height=float(p.height), 
                    yrange=self.yrange[p.tag]):
                trans = matplotlib.transforms.Affine2D()
                trans.translate(0, y)
                s.set_transform(s.get_transform() + trans)
                g.add(s)
        if vertical:
            g.rotate(90)
            g.scale(-1.0, 1.0)
        return g
    
    def asAxes(self, fig, posn, labeled=True, vertical=False):
        ax = fig.add_axes(posn)
        self.applyScaleToAxes(ax, labeled=labeled, vertical=vertical)
        g = self.makeArtist(vertical=vertical)
        ax.add_artist(g)
        return ax
        
    def applyScaleToAxes(self, ax, labeled=True, vertical=False):
        (seqaxis, trackaxis) = [ax.xaxis, ax.yaxis]
        if vertical:
            (seqaxis, trackaxis) = (trackaxis, seqaxis) 

        if not labeled:
            trackaxis.set_ticks([])
        else:
            track_positions = []
            track_labels = []
            for (y, ym, p) in self._tracks:
                if p.height > 8:
                    track_labels.append(p.label)
                    track_positions.append(ym)
            trackaxis.set_ticks(track_positions)
            trackaxis.set_ticklabels(track_labels)
            if vertical:
                for tick in trackaxis.get_major_ticks():
                    tick.label1.set_rotation('vertical')
                    tick.label2.set_rotation('vertical')
            
        seqaxis.set_major_formatter(
            matplotlib.ticker.FuncFormatter(lambda x,pos:str(int(x))))
        
        smap = self.smap.inverse()
        seq_lim = (smap.Start, smap.End)
        if vertical:
            ax.set_ylim(*seq_lim)
            ax.set_xlim(0, self.height)
        else:
            ax.set_xlim(*seq_lim)
            ax.set_ylim(0, self.height)
    
    def figureLayout(self, labeled=True, vertical=False, width=None, 
            height=None, left=None, **kw):

        if left is None:
            if labeled:
                left = max(len(p.label) for (y, ym, p) in self._tracks)
                left *= 12/72 * .5 # guess mixed chars, 12pt, inaccurate!
            else:
                left = 0
            
        height = height or self.height/72
        
        useful_width = len(self)*16/72 # ie bigish font, wide chars
        
        fkw = dict(leftovers=True, width=width, height=height, left=left, 
                useful_width=useful_width, **kw)  
        (w,h),posn,kw = rlg2mpl.figureLayout(**fkw)
        
        #points_per_base = w * posn[3] / len(self)
        if vertical:
            (w, h) = (h, w)
            posn[0:2] = reversed(posn[0:2])
            posn[2:4] = reversed(posn[2:4])
        return (w, h), posn, kw
    
    def makeFigure(self, width=None, height=None, rowlen=None, **kw):
        if rowlen:
            rows = [self[i:i+rowlen] for i in range(0, len(self), rowlen)]
        else:
            rows = [self]
            rowlen = len(self)
        kw.update(width=width, height=height)
        ((width, height), (x, y, w, h), kw) = self.figureLayout(**kw)
        N = len(rows)
        # since scales go below and titles go above, each row
        # gets the bottom margin, but not the top margin.
        vzoom = 1 + (y+h) * (N-1)
        fig = self._makeFigure(width, height * vzoom)
        for (i, row) in enumerate(rows):
            i = len(rows) - i - 1
            posn = [x, (y+i*(y+h))/vzoom, w*len(row)/rowlen, h/vzoom]
            row.asAxes(fig, posn, **kw)
        return fig
    
        
        



#!/usr/bin/env python

from string import split,strip
from cogent.struct.rna2d  import Pairs,ViennaStructure
from cogent.parse.column import column_parser

__author__ = "Shandy Wikman"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__contributors__ = ["Shandy Wikman"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Shandy Wikman"
__email__ = "ens01svn@cs.umu.se"
__status__ = "Development"

def pfold_parser(lines):
    """Parser for Pfold output
    """
    tree,lines = tree_struct_sep(lines)
    result = column_parser(lines)

    return result

def tree_struct_sep(lines):
    """Separates tree structure from rest of the data. 

    This is done to get an excepted format for the column_parser
    """
    indx = None
    for line in lines:
        if line.startswith('; ********'):
            indx = lines.index(line)+1
            break
    return lines[:indx],lines[indx:]



cimport cython
cimport numpy as np
import numpy as np
from numpy cimport npy_intp
from cogent.maths.spatial.ckd3 cimport kdpoint, points, kdnode, build_tree, rn
from stdlib cimport malloc, free

__version__ = "('1', '5', '3')"

cdef extern from "numpy/arrayobject.h":
#    cdef object PyArray_SimpleNewFromData(int nd, npy_intp *dims,\
#                                            int typenum, void *data)
    cdef void import_array()
#    cdef enum requirements:
#        NPY_OWNDATA

cdef extern from "math.h":
    float pi "M_PI"  # Aliasing the C define to "pi"

@cython.boundscheck(False)
def asa_loop(np.ndarray[DTYPE_t, ndim =2] qcoords, np.ndarray[DTYPE_t, ndim =2] lcoords,\
             np.ndarray[DTYPE_t, ndim =1] qradii,  np.ndarray[DTYPE_t, ndim =1] lradii,\
             np.ndarray[DTYPE_t, ndim =2] spoints, np.ndarray[DTYPE_t, ndim =1] box,\
             DTYPE_t probe, UTYPE_t bucket_size, MAXSYM =200000):

    # looping indexes atom, neighbor, sphere
    cdef int idx, idxn, idxn_skip, idxs, lidx, lidx_c
    # flags
    cdef int is_accessible
    # initialize variables
    cdef UTYPE_t n_acc_points
    cdef DTYPE_t qradius, lradius, search_limit
    cdef DTYPE_t lradius_max = 2.0 + probe
    #malloc'ed
    cdef DTYPE_t *rspoint = <DTYPE_t *>malloc(3 * sizeof(DTYPE_t))
    cdef DTYPE_t *distance = <DTYPE_t *>malloc(3 * sizeof(DTYPE_t))
    cdef DTYPE_t *distance_sq = <DTYPE_t *>malloc(3 * sizeof(DTYPE_t))
    cdef DTYPE_t **dstptr = <DTYPE_t **>malloc(sizeof(DTYPE_t*))
    cdef UTYPE_t **idxptr = <UTYPE_t **>malloc(sizeof(UTYPE_t*))
    # cdef DTYPE_t *areas = <DTYPE_t *>malloc(qcoords.shape[0] * sizeof(DTYPE_t))
    cdef np.ndarray[DTYPE_t, ndim=1] areas = np.ndarray((qcoords.shape[0],), dtype=np.float64)
    
    #c arrays from numpy
    cdef DTYPE_t *qcoords_c = <DTYPE_t *>qcoords.data
    cdef DTYPE_t *lcoords_c = <DTYPE_t *>lcoords.data
    cdef DTYPE_t *spoints_c = <DTYPE_t *>spoints.data
    #datas
    cdef UTYPE_t ssize = spoints.shape[0]
    cdef UTYPE_t lsize = lradii.shape[0]
    cdef npy_intp qpnts = qcoords.shape[0]
    cdef DTYPE_t const_pi = pi * 4.0 / ssize

    #pointers
    cdef UTYPE_t *ridx
    cdef UTYPE_t *ridx_div
        
    # create a temporary array of lattice points, which are within a box around
    # the query atoms. The kd-tree will be constructed from those filterd atoms.
    cdef DTYPE_t *box_c = <DTYPE_t *>box.data
    cdef DTYPE_t *t_ptr                                                      # temporary pointer
    cdef DTYPE_t *t_arr = <DTYPE_t *>malloc(3 * MAXSYM * sizeof(DTYPE_t))    # temporary array of symmetry
    cdef UTYPE_t *t_lid = <UTYPE_t *>malloc(    MAXSYM * sizeof(UTYPE_t))    # maping to original indices
    lidx_c = 0
    for 0 <= lidx < lcoords.shape[0]:
        t_ptr = lcoords_c + lidx * 3
        if  box_c[0] <= (t_ptr  )[0] <= box_c[3] and\
            box_c[1] <= (t_ptr+1)[0] <= box_c[4] and\
            box_c[2] <= (t_ptr+2)[0] <= box_c[5]:
            t_arr[3*lidx_c  ] = (t_ptr  )[0]
            t_arr[3*lidx_c+1] = (t_ptr+1)[0]
            t_arr[3*lidx_c+2] = (t_ptr+2)[0]           
            t_lid[lidx_c] = lidx
            lidx_c += 1    

    #make kd-tree
    cdef kdpoint search_point
    cdef npy_intp neighbor_number
    cdef kdpoint *kdpnts = points(t_arr, lidx_c, 3)
    cdef kdnode  *tree = build_tree(kdpnts, 0, lidx_c -1, 3, bucket_size, 0)
    
    for 0 <= idx < qpnts:
        qradius = qradii[idx]
        #print qradius, lradius_max
        search_point.coords = qcoords_c + idx*3
        #print search_point.coords[0], search_point.coords[1], search_point.coords[2]
        search_limit = (qradius + lradius_max) * (qradius + lradius_max)
        #print search_limit
        neighbor_number = <npy_intp>rn(tree, kdpnts, search_point, dstptr, idxptr, search_limit, 3, 100)
        #print neighbor_number
        #create array of real indices
        ridx = <UTYPE_t *>malloc(neighbor_number * sizeof(UTYPE_t))
        ridx_div = <UTYPE_t *>malloc(neighbor_number * sizeof(UTYPE_t))

        idxn_skip = 0
        for 0 <= idxn < neighbor_number:
            if dstptr[0][idxn] <= 0.001:
                idxn_skip = 1
                continue
            ridx[idxn - idxn_skip] = kdpnts[idxptr[0][idxn]].index * 3
            ridx_div[idxn - idxn_skip] = t_lid[kdpnts[idxptr[0][idxn]].index] % lsize

        n_acc_points = 0
        for 0 <= idxs < ssize:  # loop over sphere points
            is_accessible = 1
            #if idxs == 0:
            #print search_point.coords[0], search_point.coords[1], search_point.coords[2], '\t',
            #if idxs == 0:
            #print (spoints_c + idxs*3)[0], (spoints_c + idxs*3+1)[0], (spoints_c + idxs*3+2)[0], '\t',
            #if idxs == 0:
            #print qradius, '\t',
            rspoint[0] = search_point.coords[0] + (spoints_c + idxs*3  )[0] * qradius
            rspoint[1] = search_point.coords[1] + (spoints_c + idxs*3+1)[0] * qradius
            rspoint[2] = search_point.coords[2] + (spoints_c + idxs*3+2)[0] * qradius
            #if idxs == 0:
            #print rspoint[0], rspoint[1], rspoint[2]
            #real_point = point * qradius + qcoord
            for 0 <= idxn < neighbor_number - idxn_skip:                                # loop over neighbors
                #if dstptr[0][idxn] == 0.:
                #    continue
                #print '       ', neighbor_number, idxn_skip,idxn, ridx_div[idxn] ,ridx[idxn]
                lradius = lradii[ridx_div[idxn]]
                #print (lcoords_c + ridx[idxn]*3  )[0], (lcoords_c + ridx[idxn]*3+1)[0], (lcoords_c + ridx[idxn]*3+2)[0],
                distance[0] = rspoint[0] - (t_arr + ridx[idxn]  )[0]
                distance[1] = rspoint[1] - (t_arr + ridx[idxn]+1)[0]
                distance[2] = rspoint[2] - (t_arr + ridx[idxn]+2)[0]
                #print '\t', distance[0], distance[1], distance[2],
                distance_sq[0] = distance[0] * distance[0]
                distance_sq[1] = distance[1] * distance[1]
                distance_sq[2] = distance[2] * distance[2]
                #print '\t', distance_sq[0], distance_sq[1], distance_sq[2], lradius * lradius,
                if distance_sq[0] + distance_sq[1] + distance_sq[2] < lradius * lradius:
                    is_accessible = 0
                    #print 'NA'
                    break
                #print
            if is_accessible == 1:
                n_acc_points += 1
        #print n_acc_points
        areas[idx] = const_pi * n_acc_points * qradius * qradius
        free(ridx)
        free(ridx_div)
    free(t_arr)
    free(t_lid)
    free(distance)
    free(distance_sq)
    free(rspoint)
    free(idxptr[0])
    free(idxptr)
    free(dstptr)
    # import_array()
    # cdef np.ndarray output = PyArray_SimpleNewFromData(1, &qpnts, NPY_DOUBLE, <void*>areas)
    # output.flags = output.flags|(NPY_OWNDATA) # this sets the ownership bit
    # PyArray_FLAGS(&output)
    # print PyArray_FLAGS(output) |(NPY_OWNDATA)
    # output = PyArray_NewCopy(output, NPY_CORDER)
    return areas



#!/usr/bin/env python

from os import getcwd, remove, rmdir, mkdir, path
import tempfile, shutil
from cogent.util.unit_test import TestCase, main
from cogent.util.misc import flatten
from cogent.app.dialign import Dialign

__author__ = "Gavin Huttley"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Gavin Huttley"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"

class GeneralSetUp(TestCase):
    
    def setUp(self):
        """Dialign general setUp method for all tests"""
        self.seqs1 = ['LDTAPCLFSDGSPQKAAYVLWDQTILQQDITPLPSHETHSAQKGELLALICGLRAAK',
            'PDADHTWYTDGSSLLQEGQRKAGAAVTTETEVIWAKALDAGTSAQRAELIALTQALKM',
            'RPGLCQVFADATPTGWGLVMGHQRMRGTFSAPLPIHTAELLAACFARSRSGANIIGTDNSVV',
            'MLKQVEIFTDGSCLGNPGPGGYGAILRYRGREKTFSAGYTRTTNNRMELMAAIV']
        self.labels1 = ['>HTL2','>MMLV', '>HEPB', '>ECOL']
        self.lines1 = flatten(zip(self.labels1,self.seqs1))
        self.out = \
"""
                            DIALIGN 2.2.1 
                            *************

           Program code written by Burkhard Morgenstern and Said Abdeddaim 
              e-mail contact: dialign (at) gobics (dot) de 

           Published research assisted by DIALIGN 2 should cite:  

              Burkhard Morgenstern (1999).
              DIALIGN 2: improvement of the segment-to-segment
              approach to multiple sequence alignment.
              Bioinformatics 15, 211 - 218. 

           For more information, please visit the DIALIGN home page at 

              http://bibiserv.techfak.uni-bielefeld.de/dialign/ 

          ************************************************************



    program call:  dialign2-2 -fa -fn /tmp/di/seq1.fasta /tmp/di/seq1.txt  


    Aligned sequences:          length:
    ==================          =======

    1) HTL2                        57
    2) MMLV                        58
    3) HEPB                        62
    4) ECOL                        54

    Average seq. length:           57.8 


    Please note that only upper-case letters are considered to be aligned. 


    Alignment (DIALIGN format):
    ===========================

 HTL2               1   ldtapC-LFS DGS------P QKAAYVL--- ----WDQTIL QQDITPLPSH 
 MMLV               1   pdadhtw-YT DGSSLLQEGQ RKAGAAVtte teviWa---- KALDAG---T 
 HEPB               1   rpgl-CQVFA DAT------P TGWGLVM--- ----GHQRMR GTFSAPLPIH 
 ECOL               1   mlkqv-EIFT DGSCLGNPGP GGYGAIL--- ----RYRGRE KTFSAGytrT 

                        0000000588 8882222229 9999999000 0000666666 6666633334 

 HTL2              37   ethSAQKGEL LALICGLRAa k--------- --- 
 MMLV              43   ---SAQRAEL IALTQALKm- ---------- --- 
 HEPB              37   t------AEL LAA-CFARSr sganiigtdn svv 
 ECOL              43   ---TNNRMEL MAAIv----- ---------- --- 

                        0003333455 5533333300 0000000000 000 




    Sequence tree:
    ==============

 Tree constructed using UPGMAbased on DIALIGN fragment weight scores

 ((HTL2        :0.130254MMLV        :0.130254):0.067788(HEPB        :0.120520ECOL        :0.120520):0.077521);



"""
        self.temp_dir = tempfile.mkdtemp()
        try:
            #create sequence files
            f = open(path.join(self.temp_dir, 'seq1.txt'),'w')
            f.write('\n'.join(self.lines1))
            f.close()
        except OSError:
            pass

class DialignTests(GeneralSetUp):
    """Tests for the Dialign application controller"""
    
    def test_base_command(self):
        """Dialign BaseCommand should return the correct BaseCommand"""
        c = Dialign()
        self.assertEqual(c.BaseCommand,
            ''.join(['cd ','"%s/"; ' % getcwd(),'dialign2-2']))
        
        c.Parameters["-fa"].on()
        self.assertEqual(c.BaseCommand,\
            ''.join(['cd ','"%s/"; ' % getcwd(),'dialign2-2 -fa']))
    
    def test_align(self):
        """test aligning samples"""
        c = Dialign(WorkingDir=self.temp_dir,
            params={"-fn":path.join(self.temp_dir,"seq1.txt")})
        c.Parameters["-fa"].on()
        res = c(path.join(self.temp_dir, 'seq1.txt'))
        align = "".join(res["Align"].readlines())
    
    def test_cleaning_up(self):
        """not a test, just removes the temp files"""
        shutil.rmtree(self.temp_dir)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

from os                    import remove
from cogent.util.unit_test import TestCase, main
from cogent.core.info      import Info
from cogent.app.dynalign   import Dynalign

__author__ = "Shandy Wikman"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__contributors__ = ["Shandy Wikman"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Shandy Wikman"
__email__ = "ens01svn@cs.umu.se"
__status__ = "Development"

class DynalignTest(TestCase):
    """Tests for Dynalign application controller"""

    def setUp(self):
        self.input1 = dynalign_input1
        self.input2 = dynalign_input2
        
        
    def test_stdout_input_as_lines(self):
        """Test Dynalign stdout input as lines"""

        d = Dynalign(InputHandler='_input_as_lines')
        exp = '%s\n' % '\n'.join([str(i).strip('\n') for i in dynalign_stdout])

        res = d([self.input1,self.input2])
        obs = res['StdOut'].read()
        self.assertEqual(obs,exp)
        res.cleanUp()

    def test_stdout_input_as_string(self):
        """Test Dynalign stdout input as string"""

        d = Dynalign()
        exp = '%s\n' % '\n'.join([str(i).strip('\n') for i in dynalign_stdout])

        f = open('/tmp/dyn1','w')
        txt = '\n'.join([str(i).strip('\n') for i in self.input1])
        f.write(txt)
        f.close()
        s = open('/tmp/dyn2','w')
        txt = '\n'.join([str(i).strip('\n') for i in self.input2])
        s.write(txt)
        s.close()
        res = d(['/tmp/dyn1','/tmp/dyn2'])
        obs = res['StdOut'].read()
        self.assertEqual(obs,exp)
        res.cleanUp()

    def test_get_result_path(self):
        """Tests Dynalign result path"""

        d = Dynalign(InputHandler='_input_as_lines')
        res = d([self.input1,self.input2])
        self.assertEqualItems(res.keys(),['StdOut','StdErr','ExitStatus',\
        'seq_1_ct','seq_2_ct','alignment'])
        self.assertEqual(res['ExitStatus'],0)
        res.cleanUp()

dynalign_input1 = [';\n', 'testSeq1\n', '\n', 
'GGCTAGATAGCTCAGGTCGGTTCGATCCCGGCTCTGGCC1']
dynalign_input2 = [';\n', 'testSeq2\n', '\n', 
'GGCTAGATAGCTGTGTCGTCGGTTCGATCCCGGCTCTGGCC1']
dynalign_stdout = ['12%\n', '25%\n', '38%\n', '51%\n', '64%\n', '76%\n', 
'89%\n']


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import numpy
import warnings

from cogent.recalculation.definition import PositiveParamDefn, RatioParamDefn, \
        CalculationDefn, MonotonicDefn, ProductDefn, ConstDefn, PartitionDefn, \
        NonParamDefn, CallDefn, SelectForDimension, \
        GammaDefn, WeightedPartitionDefn, CalcDefn
from cogent.maths.matrix_exponentiation import PadeExponentiator, \
        FastExponentiator, CheckedExponentiator, LinAlgError

__author__ = "Peter Maxwell"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Peter Maxwell", "Gavin Huttley"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Peter Maxwell"
__email__ = "pm67nz@gmail.com"
__status__ = "Production"

# Custom subclasses of Defn (see cogent.recalulation) for use by substitution models.

class AlignmentAdaptDefn(CalculationDefn):
    name = 'leaf_likelihoods'
    def calc(self, model, alignment):
        return model.convertAlignment(alignment)       

class LengthDefn(PositiveParamDefn):
    name = 'length'
    valid_dimensions = ('edge',)
    independent_by_default = True
    upper = 10.0

class RateDefn(RatioParamDefn):
    name = 'rate'
    valid_dimensions = ('bin', 'locus')
    independent_by_default = True
    lower = 1e-3
    upper = 1e+3

class SubstitutionParameterDefn(RatioParamDefn):
    valid_dimensions = ('edge', 'bin', 'locus')
    independent_by_default = False

class ExpDefn(CalculationDefn):
    name = 'exp'
    
    def calc(self, expm):
        (allow_eigen, check_eigen, allow_pade) = {
            'eigen': (True, False, False),
            'checked': (True, True, False),
            'pade': (False, False, True),
            'either': (True, True, True),
            }[str(expm)]
        
        if not allow_eigen:
            return PadeExponentiator
        
        eigen = CheckedExponentiator if check_eigen else FastExponentiator
        
        if not allow_pade:
            return eigen
        else:
            def _both(Q, eigen=eigen):
                try:
                    return eigen(Q)
                except (ArithmeticError, LinAlgError), detail:
                    if not _both.given_expm_warning:
                        warnings.warn("using slow exponentiator because '%s'"
                                % str(detail))
                        _both.given_expm_warning = True
                    return PadeExponentiator(Q)
            _both.given_expm_warning = False
            return _both





#!/usr/bin/env python
from numpy import concatenate, repeat, array, zeros, histogram, arange, uint, zeros
from numpy.random import permutation, randint, sample, multinomial
from random import Random, _ceil, _log

"""Given array of objects (counts or indices), perform rarefaction analyses."""

__author__ = "Sandra Smit"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Rob Knight"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Development"


class MyRandom(Random):
    """Adding a method to sample from array only"""
    def sample_array(self, population, k):
        """Chooses k unique random elements from a population sequence.

        Returns a new list containing elements from the population while
        leaving the original population unchanged.  The resulting list is
        in selection order so that all sub-slices will also be valid random
        samples.  This allows raffle winners (the sample) to be partitioned
        into grand prize and second place winners (the subslices).

        Members of the population need not be hashable or unique.  If the
        population contains repeats, then each occurrence is a possible
        selection in the sample.

        To choose a sample in a range of integers, use xrange as an argument.
        This is especially fast and space efficient for sampling from a
        large population:   sample(xrange(10000000), 60)
        """

        # Sampling without replacement entails tracking either potential
        # selections (the pool) in a list or previous selections in a set.

        # When the number of selections is small compared to the
        # population, then tracking selections is efficient, requiring
        # only a small set and an occasional reselection.  For
        # a larger number of selections, the pool tracking method is
        # preferred since the list takes less space than the
        # set and it doesn't suffer from frequent reselections.

        n = len(population)
        if not 0 <= k <= n:
            raise ValueError("sample larger than population")
        random = self.random
        _int = int
        result = zeros(k)
        setsize = 21        # size of a small set minus size of an empty list
        if k > 5:
            setsize += 4 ** _ceil(_log(k * 3, 4)) # table size for big sets
        if n <= setsize or hasattr(population, "keys"):
            # An n-length list is smaller than a k-length set, or this is a
            # mapping type so the other algorithm wouldn't work.
            pool = array(list(population))
            for i in xrange(k):         # invariant:  non-selected at [0,n-i)
                j = _int(random() * (n-i))
                result[i] = pool[j]
                pool[j] = pool[n-i-1]   # move non-selected item into vacancy
        else:
            try:
                selected = set()
                selected_add = selected.add
                for i in xrange(k):
                    j = _int(random() * n)
                    while j in selected:
                        j = _int(random() * n)
                    selected_add(j)
                    result[i] = population[j]
            except (TypeError, KeyError):   # handle (at least) sets
                if isinstance(population, list):
                    raise
                return self.sample_array(tuple(population), k)
        return result

_inst = MyRandom()
sample = _inst.sample_array


def subsample(counts, n):
    """Subsamples new vector from vector of orig items.
    
    Returns all items if requested sample is larger than number of items.
    """
    if counts.sum() <= n:
        return counts
    nz = counts.nonzero()[0]
    unpacked = concatenate([repeat(array(i,), counts[i]) for i in nz])
    permuted = permutation(unpacked)[:n]
    result = zeros(len(counts))
    for p in permuted:
        result[p] += 1
    return result

def subsample_freq_dist_nonzero(counts, n, dtype=uint):
    """Subsamples new vector from vector of orig items.

    Returns all items if requested sample is larger than number of items.

    This version uses the cumsum/frequency distribution method.
    """
    if counts.sum() <= n:
        return counts
    result = zeros(len(counts), dtype=dtype)
    nz = counts.nonzero()[0]
    compressed = counts.take(nz)
    sums = compressed.cumsum()
    total = sums[-1]
    del compressed
    curr = n
    while curr:
        pick = randint(0, total)
        #print pick, sums, sums.searchsorted(pick), '\n'
        index = sums.searchsorted(pick,side='right')
        result[nz[index]] += 1
        sums[index:] -= 1
        curr -= 1
        total -= 1
    return result

def subsample_random(counts, n, dtype=uint):
    """Subsamples new vector from vector of orig items.

    Returns all items if requested sample is larger than number of items.

    This version uses random.sample.
    """
    if counts.sum() <= n:
        return counts
    nz = counts.nonzero()[0]
    unpacked = concatenate([repeat(array(i,), counts[i]) for i in nz])
    permuted = sample(unpacked, n)
    result = zeros(len(counts),dtype=dtype)
    for p in permuted:
        result[p] += 1
    return result

def subsample_multinomial(counts, n, dtype=None):
    """Subsamples new vector from vector of orig items.

    Returns all items if requested sample is larger than number of items.

    This version uses the multinomial to sample WITH replacement.
    """
    if dtype == None:
        dtype=counts.dtype
    if counts.sum() <= n:
        return counts
    result = zeros(len(counts), dtype=dtype)
    nz = counts.nonzero()[0]
    compressed = counts.take(nz).astype(float)
    compressed /= compressed.sum()
    result = multinomial(n, compressed).astype(dtype)
    counts[nz] = result
    return counts

def naive_histogram(vals, max_val=None, result=None):
    """Naive histogram for performance testing vs. numpy's.
    
    Apparently numpy's is 3x faster (non-cumulative) for larger step sizes
    (e.g. 1000) and 10x slower for small step sizes (e.g. 1), so will use
    logic to switch over depending on conditions.
    """
    if max_val is None:
       max_val = vals.max()
    if result is None:
        result = zeros(max_val+1, dtype=int)
    for v in vals:
        result[v] += 1
    return result

def wrap_numpy_histogram(max_val):
    """return convenience wrapper for numpy histogram"""
    bins = arange(max_val+2, dtype = int)   #+1 for length, +1 for leading 0
    def f(vals, max_val='ignored'): return histogram(vals, bins)[0]
    return f

def rarefaction(data, start=0, stop=None, stride=1, histogram_f=None, \
    permutation_f=permutation, is_counts=True):
    """Yields successive subsamples as vectors from vector of orig items.
    
    data can either be array of counts or array of observations. Default is
    to assume counts; set is_counts to False if this is not the case for your
    input.

    Returns all items if requested sample is larger than number of items.

    WARNING: each successive result is written into the same object (for
    convenience) so if you want the actual vectors for each rarefaction you
    need to do something like res = [r.copy() for r in rarefaction(params)].
    """
    if is_counts:   #need to transform data into indices
        nz = array(data).nonzero()[0]
        indices = concatenate([repeat(array(i,), data[i]) for i in nz])
    else:
        indices = array(data)

    if stop is None:
        stop = len(indices)
    if not stride:
        stride = 1  #avoid zero or None as stride
    max_val=indices.max() 
    if histogram_f is None:
        if stride < 100:
            histogram_f = naive_histogram
        else:
            histogram_f = wrap_numpy_histogram(max_val)
    permuted = permutation_f(indices)
    result = zeros(max_val+1, dtype=int)
    while start < stop:
        curr_slice = permuted[start:start+stride]
        result += histogram_f(curr_slice, max_val=max_val)
        yield result
        start += stride




#!/usr/bin/env python

from cogent.util.unit_test import TestCase, main
from cogent.parse.tree import DndParser
from cogent.seqsim.tree import RangeNode, balanced_breakpoints, BalancedTree, \
    RandomTree, CombTree, StarTree, LineTree
from cogent.core.usage import DnaPairs
from copy import deepcopy
from operator import mul, or_, add
from numpy import array, average, diag
from numpy.random import random, randint
from cogent.seqsim.usage import Rates

__author__ = "Rob Knight"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Rob Knight", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

class treeTests(TestCase):
    """Tests for top-level functions."""
    def test_init(self):
        """Make sure keyword arguments are being passed to baseclass"""
        node = RangeNode(LeafRange=1, Id=2, Name='foo', Length=42)
        self.assertEqual(node.LeafRange, 1)
        self.assertEqual(node.Id, 2)
        self.assertEqual(node.Name, 'foo')
        self.assertEqual(node.Length, 42)

    def test_balanced_breakpoints(self):
        """balanced_breakpoints should produce expected arrays."""
        self.assertRaises(ValueError, balanced_breakpoints, 1)
        self.assertEqual(balanced_breakpoints(2), array([0]))
        self.assertEqual(balanced_breakpoints(4), array([1,0,2]))
        self.assertEqual(balanced_breakpoints(8), \
            array([3,1,5,0,2,4,6]))
        self.assertEqual(balanced_breakpoints(16), \
            array([7,3,11,1,5,9,13,0,2,4,6,8,10,12,14]))
        self.assertEqual(balanced_breakpoints(32), \
            array([15,7,23,3,11,19,27,1,5,9,13,17,21,25,29,\
            0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30]))

    def test_BalancedTree(self):
        """BalancedTree should return a balanced tree"""
        b = BalancedTree(4)
        self.assertEqual(len(list(b.traverse())), 4)
        b.assignIds()
        self.assertEqual(str(b), '((0,1)4,(2,3)5)6')

    def test_RandomTree(self):
        """RandomTree should return correct number of nodes
        
        NOTE: all the work is done in breakpoints, which is thoroughly
        tested indepndently. RandomTree just makes permutations.
        """
        d = {}
        for i in range(10):
            r = RandomTree(100)
            self.assertEqual(len(list(r.traverse())), 100)
            r.assignIds()
            #make sure we get different trees each time...
            s = str(r)
            assert s not in d
            d[s] = None

    def test_CombTree(self):
        """CombTree should return correct topology"""
        c = CombTree(4)
        c.assignIds()
        self.assertEqual(str(c), '(0,(1,(2,3)4)5)6')
        c = CombTree(4, deepest_first=False)
        c.assignIds()
        self.assertEqual(str(c), '(((0,1)4,2)5,3)6')

    def test_StarTree(self):
        """StarTree should return correct star topology and # nodes"""
        t = StarTree(5)
        self.assertEqual(len(t.Children), 5)
        for c in t.Children:
            assert c.Parent is t

    def test_LineTree(self):
        """LineTree should return correct number of nodes"""
        t = LineTree(5)
        depth = 1
        curr = t
        while curr.Children:
            self.assertEqual(len(curr.Children), 1)
            depth += 1
            curr = curr.Children[0]
        self.assertEqual(depth, 5)

class RangeTreeTests(TestCase):
    """Tests of the RangeTree class."""
    def setUp(self):
        """Make some standard objects to test."""
        #Notes on sample string:
        #
        #1. trailing zeros are stripped in conversion to/from float, so result
        #   is only exactly the same without them.
        #
        #2. trailing chars (e.g. semicolon) are not recaptured in the output,
        #   so were deleted from original Newick-format string.
        #
        #3. whitespace is stripped, but is handy for formatting, so is stripped
        #   from original string before comparisons.
        self.sample_tree_string = """
    (
    (
    xyz:0.28124,
    (
    def:0.24498,
    mno:0.03627)
    A:0.1771)
    B:0.0487,

    abc:0.05925,
    (
    ghi:0.06914,
    jkl:0.13776)
    C:0.09853)
    """
        self.t = DndParser(self.sample_tree_string, RangeNode)
        self.i = self.t.indexByAttr('Name')
        
        self.sample_string_2 = '((((a,b),c),(d,e)),((f,g),h))'
        self.t2 = DndParser(self.sample_string_2, RangeNode)
        self.i2 = self.t2.indexByAttr('Name')

        self.sample_string_3 = '(((a,b),c),(d,e))'
        self.t3 = DndParser(self.sample_string_3, RangeNode)

    def test_str(self):
        """RangeNode should round-trip Newick string corrrectly."""

        r = RangeNode()
        self.assertEqual(str(r), '()')
        
        #should work for tree with branch lengths set
        t = DndParser(self.sample_tree_string, RangeNode)
        expected = self.sample_tree_string.replace('\n', '')
        expected = expected.replace(' ', '')
        self.assertEqual(str(t), expected)
        #self.assertEqual(t.getNewick(with_distances=True), expected)
        #should also work for tree w/o branch lengths
        t2 = DndParser(self.sample_string_2, RangeNode)
        self.assertEqual(str(t2), self.sample_string_2)

    def test_traverse(self):
        """RangeTree traverse should visit all nodes in correct order"""
        t = self.t
        i = self.i
        #first, check that lengths are correct
        #naked traverse() only does leaves; should be 6.
        self.assertEqual(len(list(t.traverse())), 6)
        #traverse() with self_before should count all nodes.
        self.assertEqual(len(list(t.traverse(self_before=True))), 10)
        #traverse() with self_after should have same count as self_before
        self.assertEqual(len(list(t.traverse(self_after=True))), 10)
        #traverse() with self_before and self_after should visit internal
        #nodes multiple times
        self.assertEqual(len(list(t.traverse(True,True))), 14)
        
        #now, check that items are in correct order
        exp = ['xyz','def','mno','abc','ghi','jkl']
        obs = [i.Name for i in t.traverse()]
        self.assertEqual(obs, exp)

        exp = [None, 'B', 'xyz', 'A', 'def', 'mno', 'abc', 'C', 'ghi', 'jkl']
        obs = [i.Name for i in t.traverse(self_before=True)]
        self.assertEqual(obs, exp)

        exp = ['xyz', 'def', 'mno', 'A', 'B', 'abc', 'ghi', 'jkl', 'C', None]
        obs = [i.Name for i in t.traverse(self_after=True)]
        self.assertEqual(obs, exp)

        exp = [None, 'B', 'xyz', 'A', 'def', 'mno', 'A', 'B', 'abc', 'C', \
            'ghi', 'jkl', 'C', None]
        obs = [i.Name for i in t.traverse(self_before=True, self_after=True)]
        self.assertEqual(obs, exp)

    def test_indexByAttr(self):
        """RangeNode indexByAttr should make index using correct attr"""
        t = self.t
        i = self.i
        #check that we got the right number of elements
        #all elements unique, so should be same as num nodes
        self.assertEqual(len(i), len(list(t.traverse(self_before=True))))
        #check that we got everything
        i_keys = i.keys()
        i_vals = i.values()
        for node in t.traverse(self_before=True):
            assert node.Name in i_keys
            assert node in i_vals
            #can't predict which node will have None as the key
            if node.Name is not None:
                assert i[node.Name] is node
        #check that it works when elements are not unique
        t = self.t3
        for node in t.traverse(self_before=True):
            node.X = 'b'
        for node in t.traverse():
            node.X = 'a'
        result = t.indexByAttr('X', multiple=True)
        self.assertEqual(len(result), 2)
        self.assertEqual(len(result['a']), 5)
        self.assertEqual(len(result['b']), 4)
        for n in t.traverse():
            assert n in result['a']
            assert not n in result['b']
        
    def test_indexbyFunc(self):
        """RangeNode indexByFunc should make index from function"""
        t = self.t
        def f(n):
            try:
                return n.Name.isupper()
            except AttributeError:
                return None
            
        i = self.i
        f_i = t.indexByFunc(f)
        self.assertEqual(len(f_i), 3)
        self.assertEqual(f_i[True], [i['B'], i['A'], i['C']])
        self.assertEqual(f_i[False], [i['xyz'], i['def'], i['mno'], \
            i['abc'], i['ghi'], i['jkl']])
        self.assertEqual(f_i[None], [i[None]])

    def test_assignIds(self):
        """RangeNode assignIds should work as expected"""
        t = self.t2
        index = self.i2
        
        t.assignIds()
        #check that ids were set correctly on the leaves
        for i, a in enumerate('abcdefgh'):
            self.assertEqual(index[a].Id, i)
        #check that ranges were set correctly on the leaves
        for i, a in enumerate('abcdefgh'):
            self.assertEqual(index[a].LeafRange, (i, i+1))

        #check that internal ids were set correctly
        obs = [i.Id for i in t.traverse(self_after=True)]
        exp = [0,1,8,2,9,3,4,10,11,5,6,12,7,13,14]
        self.assertEqual(obs, exp)

        #check that internal ranges were set correctly
        obs = [i.LeafRange for i in t.traverse(self_after=True)]
        exp = [(0,1),(1,2),(0,2),(2,3),(0,3),(3,4),(4,5),(3,5),(0,5), \
            (5,6),(6,7),(5,7),(7,8),(5,8),(0,8)]
        self.assertEqual(obs, exp)

    def test_propagateAttr(self):
        """RangeNode propagateAttr should send attr down tree, unless set"""
        t = self.t
        i = self.i
        for n in t.traverse(self_before=True):
            assert not hasattr(n, 'XYZ')
        t.XYZ = 3
        t.propagateAttr('XYZ')
        for n in t.traverse(self_before=True):
            self.assertEqual(n.XYZ, 3)
        #shouldn't overwrite internal nodes by default
        a_children = list(i['A'].traverse(self_before=True))
        i['A'].GHI = 5
        t.GHI = 1
        t.propagateAttr('GHI')
        for n in t.traverse(self_before=True):
            if n in a_children:
                self.assertEqual(n.GHI, 5)
            else:
                self.assertEqual(n.GHI, 1)

        t.GHI = 0
        t.propagateAttr('GHI', overwrite=True)
        for n in t.traverse(self_before=True):
            self.assertEqual(n.GHI, 0)
        
    def test_delAttr(self):
        """RangeNode delAttr should delete attr from self and children"""
        t = self.t2
        for n in t.traverse(self_before=True):
            assert hasattr(n, 'Name')
        t.delAttr('Name')
        for n in t.traverse(self_before=True):
            assert not hasattr(n, 'Name')

    def test_accumulateAttr(self):
        """RangeNode accumulateAttr should accumulate attr in right direction"""
        t = self.t3
        #test towards_leaves (the default)
        f = lambda a, b: b + 1
        for n in t.traverse(self_before=True):
            n.Level = 0
        t.accumulateAttr('Level', f=f)
        levels = [i.Level for i in t.traverse(self_before=True)]
        self.assertEqual(levels, [0,1,2,3,3,2,1,2,2])
        for n in t.traverse(self_before=True):
            n.Level=0
        #test away from leaves
        f = lambda a, b : max(a, b+1)
        for n in t.traverse(self_before=True):
            n.Level=0
        t.accumulateAttr('Level', towards_leaves=False,f=f)
        levels = [i.Level for i in t.traverse(self_before=True)]
        self.assertEqual(levels, [3,2,1,0,0,0,1,0,0])
    
    def test_accumulateChildAttr(self):
        """RangeNode accumulateChildAttr should work as expected"""
        t = self.t2
        i = self.i2
        i['a'].x = 3
        i['b'].x = 4
        i['d'].x = 0
        i['f'].x = 1
        i['g'].x = 1
        i['h'].x = 1

        t.accumulateChildAttr('x', f=mul)
        self.assertEqual([i.__dict__.get('x', None) for i in \
            t.traverse(self_after=True)],
            [3, 4, 12, None, 12, 0, None, 0, 0, 1, 1, 1, 1, 1, 0])
        
        t.accumulateChildAttr('x', f=add)
        self.assertEqual([i.__dict__.get('x', None) for i in \
            t.traverse(self_after=True)],
            [3, 4, 7, None, 7, 0, None, 0, 7, 1, 1, 2, 1, 3, 10])
       
    def test_assignLevelsFromRoot(self):
        """RangeNode assignLevelsFromRoot should match hand-calculated levels"""
        t = self.t3
        t.assignLevelsFromRoot()
        levels = [i.Level for i in t.traverse(self_before=True)]
        self.assertEqual(levels, [0,1,2,3,3,2,1,2,2])
     
    def test_assignLevelsFromLeaves(self):
        """RangeNode assignLevelsFromLeaves should match hand-calculated levels"""
        t = self.t3
        t.assignLevelsFromLeaves()
        levels = [i.Level for i in t.traverse(self_before=True)]
        self.assertEqual(levels, [3,2,1,0,0,0,1,0,0])
        t.assignLevelsFromLeaves(use_min=True)
        levels = [i.Level for i in t.traverse(self_before=True)]
        self.assertEqual(levels, [2,1,1,0,0,0,1,0,0])

    def test_attrToList(self):
        """RangeNode attrToList should return correct list of attr"""
        t = self.t3
        t.assignIds()
        t.assignLevelsFromRoot()
        #make sure nodes are in the order we expect
        self.assertEqual([n.Id for n in t.traverse(self_before=True)],
            [8,6,5,0,1,2,7,3,4])
        #by default, should return list containing all nodes
        obs = t.attrToList('Level')
        self.assertEqual(obs, [3,3,2,2,2,2,1,1,0])
        #should be able to do leaves only if specified
        obs = t.attrToList('Level', leaves_only=True)
        self.assertEqual(obs, [3,3,2,2,2])
        #should be able to specify larger size
        obs=t.attrToList('Level', size=12)
        self.assertEqual(obs, [3,3,2,2,2,2,1,1,0,None,None,None])
        #should be able to set default
        obs=t.attrToList('Level', default='x', size=12)
        self.assertEqual(obs, [3,3,2,2,2,2,1,1,0,'x','x','x'])

    def test_attrFromList(self):
        """RangeNode attrFromList should set values correctly"""
        t = self.t3
        t.assignIds()
        #by default, should set all nodes from array
        t.attrFromList('Level', [3,3,2,2,2,2,1,1,0])
        self.assertEqual([n.Level for n in t.traverse(self_before=True)], \
            [0,1,2,3,3,2,1,2,2])
        #should also work if we choose to set only the leaves (rest should
        #stay at default values)
        t.Level = -1
        t.propagateAttr('Level', overwrite=True)
        t.attrFromList('Level', [3,3,2,2,2,2,1,1,0], leaves_only=True)
        self.assertEqual([n.Level for n in t.traverse(self_before=True)], \
            [-1,-1,-1,3,3,2,-1,2,2])
        
    def test_toBreakpoints(self):
        """RangeNode toBreakpoints should give expected list"""
        t = self.t2
        t.assignIds()
        self.assertEqual(t.toBreakpoints(), [4,2,1,0,3,6,5])

    def test_fromBreakpoints(self):
        """RangeNode fromBreakpoints should have correct topology"""
        breakpoints = [4,2,1,0,3,6,5]
        t = RangeNode.fromBreakpoints(breakpoints)
        #check number of leaves
        self.assertEqual(len(list(t.traverse())), 8)
        self.assertEqual(len(list(t.traverse(self_before=True))), 15)
        #check that leaves were created in right order
        self.assertEqual([i.Id for i in t.traverse()], range(8))
        #check that whole toplogy is right wrt ids...
        nodes = list(t.traverse(self_before=True))
        obs = [i.Id for i in nodes]
        exp = [8, 9, 11, 13, 0, 1, 2, 12, 3, 4, 10, 14, 5, 6, 7]
        self.assertEqual(obs, exp)
        #...and ranges
        obs = [i.LeafRange for i in nodes]
        exp = [(0,8),(0,5),(0,3),(0,2),(0,1),(1,2),(2,3),(3,5),(3,4),(4,5), \
            (5,8),(5,7),(5,6),(6,7),(7,8)]
        self.assertEqual(obs, exp)

    def test_leafLcaDepths(self):
        """RangeNode leafLcaDepths should return expected depths"""
        t = self.t3
        result = t.leafLcaDepths()
        self.assertEqual(result, array([[0,1,2,3,3],
                                        [1,0,2,3,3],
                                        [2,2,0,3,3],
                                        [3,3,3,0,1],
                                        [3,3,3,1,0]]))

    def test_randomNode(self):
        """RandomNode should hit all nodes equally"""
        t = self.t3
        result = {}
        for i in range(100):
            ans = id(t.randomNode())
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 9)
        for node in t.traverse(self_before=True):
            assert id(node) in result

    def test_randomLeaf(self):
        """RandomLeaf should hit all leaf nodes equally"""
        t = self.t3
        result = {}
        for i in range(100):
            ans = id(t.randomLeaf())
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 5)
        for node in t.traverse():
            assert id(node) in result

    def test_randomNodeWithNLeaves(self):
        """RandomNodeWithNLeaves should return node with correct # leaves"""
        t = self.t3
        #check that only the root gets selected with 5 leaves
        result = {}
        for i in range(20):
            ans = id(t.randomNodeWithNLeaves(5))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 1)
        assert id(t) in result
        #check that nothing has 6 or 4 (for this tree) leaves
        self.assertRaises(KeyError, t.randomNodeWithNLeaves, 6)
        self.assertRaises(KeyError, t.randomNodeWithNLeaves, 4)
        #check that it works with fewer than 5 leaves
        #SINGLE LEAF:
        result = {}
        for i in range(40):
            ans = id(t.randomNodeWithNLeaves(1))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 5)
        self.assertEqual(sum(result.values()), 40)
        #TWO LEAVES:
        result = {}
        for i in range(20):
            ans = id(t.randomNodeWithNLeaves(2))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 2)
        self.assertEqual(sum(result.values()), 20)
        #THREE LEAVES:
        result = {}
        for i in range(20):
            ans = id(t.randomNodeWithNLeaves(3))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 1)
        self.assertEqual(sum(result.values()), 20)

    def test_randomNodeAtLevel(self):
        """RangeNode randomNodeAtLevel should return random node at correct level"""
        t = self.t3
        #LEAVES:
        result = {}
        for i in range(40):
            ans = id(t.randomNodeAtLevel(0))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 5)
        self.assertEqual(sum(result.values()), 40)
        
        #BACK ONE LEVEL:
        result = {}
        for i in range(20):
            ans = id(t.randomNodeAtLevel(1))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 2)
        self.assertEqual(sum(result.values()), 20)
         
        #BACK TWO LEVELS:
        result = {}
        for i in range(20):
            ans = id(t.randomNodeAtLevel(2))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 1)
        self.assertEqual(sum(result.values()), 20)
             
        #BACK THREE LEVELS (to root):
        result = {}
        for i in range(20):
            ans = id(t.randomNodeAtLevel(3))
            if ans not in result:
                result[ans] = 0
            result[ans] += 1
        self.assertEqual(len(result), 1)
        self.assertEqual(sum(result.values()), 20)
        self.assertEqual(result.keys()[0], id(t))
         

    def test_outgroupLast(self):
        """RangeNode outgroupLast should reorder nodes to put outgroup last"""
        t = self.t3
        a, b, c, d, e = t.traverse()
        self.assertEqual(t.outgroupLast(c,a,b), (a, b, c))
        self.assertEqual(t.outgroupLast(c,b,a), (b, a, c))
        self.assertEqual(t.outgroupLast(b,d,a), (b, a, d))
        self.assertEqual(t.outgroupLast(c,d,e), (d, e, c))
        self.assertEqual(t.outgroupLast(a,d,e), (d, e, a))
        self.assertEqual(t.outgroupLast(a,d,b), (a, b, d))
        #check that it works if we suppress the cache
        self.assertEqual(t.outgroupLast(c,a,b, False), (a, b, c))
        self.assertEqual(t.outgroupLast(c,b,a, False), (b, a, c))
        self.assertEqual(t.outgroupLast(b,d,a, False), (b, a, d))
        self.assertEqual(t.outgroupLast(c,d,e, False), (d, e, c))
        self.assertEqual(t.outgroupLast(a,d,e, False), (d, e, a))
        self.assertEqual(t.outgroupLast(a,d,b, False), (a, b, d))

    def test_filter(self):
        """RangeNode filter should keep or omit selected nodes."""
        t_orig = self.t2
        
        t = deepcopy(t_orig)
        idx = t.indexByAttr('Name')
        to_keep = map(idx.__getitem__, 'abch')
        curr_leaves = list(t.traverse())
        t.filter(to_keep)
        curr_leaves = list(t.traverse())
        for i in to_keep:
            assert i in curr_leaves
        for i in map(idx.__getitem__, 'defg'):
            assert i not in curr_leaves
        #note that it collapses one-child nodes
        self.assertEqual(str(t), '(((a,b),c),h)')

        #test same thing but omitting
        t = deepcopy(t_orig)
        idx = t.indexByAttr('Name')
        to_omit = map(idx.__getitem__, 'abch')
        t.filter(to_omit, keep=False)
        curr_leaves = list(t.traverse())
        for i in to_omit:
            assert i not in curr_leaves
        for i in map(idx.__getitem__, 'defg'):
            assert i in curr_leaves
        #note that it collapses one-child nodes
        self.assertEqual(str(t), '((d,e),(f,g))')

        #test that it works with internal nodes
        t = deepcopy(t_orig)
        idx = t.indexByAttr('Name')
        to_omit = [idx['a'].Parent.Parent]
        t.filter(to_omit, keep=False)
        self.assertEqual(str(t), '((d,e),((f,g),h))')

        #test that it adds branch lengths
        t = deepcopy(t_orig)
        idx = t.indexByAttr('Name')
        for i in t.traverse(self_after=True):
            i.BranchLength = 1
        to_omit = map(idx.__getitem__, 'abdefg')
        t.filter(to_omit, keep=False)
        self.assertEqual(str(t), '(c,h)')

        #test that it got rid of the temporary '_selected' attribute
        for node in t.traverse(self_before=True):
            assert not hasattr(node, '_selected')

        #if nothing valid in to_keep, should return empty tree
        t = deepcopy(t_orig)
        idx = t.indexByAttr('Name')
        to_keep = []
        t.filter(to_keep, keep=True)
        curr_leaves = list(t.traverse())
        assert len(curr_leaves), 0

        #if nothing valid in to_keep, should return empty tree
        t = deepcopy(t_orig)
        idx = t.indexByAttr('Name')
        to_keep = list('abcde')     #note: just labels, not nodes
        t.filter(to_keep, keep=True)
        curr_leaves = list(t.traverse())
        assert len(curr_leaves), 0

    def test_addChildren(self):
        """RangeNode add_children should add specified # children to list"""
        t = RangeNode()
        t2 = RangeNode(Parent=t)
        t.addChildren(5)
        self.assertEqual(len(t.Children), 6)
        assert t.Children[0] is t2
        for c in t.Children:
            assert c.Parent is t

class OldPhyloNodeTests(TestCase):
    """Tests of the PhyloNode class -- these are all now methods of RangeNode."""
    def setUp(self):
        """Make a couple of standard trees"""
        self.t1 = DndParser('((a,(b,c)),(d,e))', RangeNode)
        #selt.t1 indices: ((0,(1,2)5)6,(3,4)7)8
    
    def test_makeIdIndex(self):
        """RangeNode makeIdIndex should assign ids to every node"""
        self.t1.makeIdIndex()
        result = self.t1.IdIndex
        nodes = list(self.t1.traverse(self_before=True))
        #check we got an entry for each node
        self.assertEqual(len(result), len(nodes))
        #check the ids are in the result
        for i in nodes:
            assert hasattr(i, 'Id')
            assert i.Id in result
            
    def test_assignQ_single_passed(self):
        """RangeNode assignQ should propagate single Q param down tree"""
        #should work if Q explicitly passed
        t = self.t1
        Q = ['a']
        t.assignQ(Q)
        for node in t.traverse(self_before=True):
            assert node.Q is Q

    def test_assignQ_single_set(self):
        """RangeNode assignQ should propagate single Q if set"""
        t = self.t1
        Q = ['a']
        assert not hasattr(t, 'Q')
        t.Q = Q
        t.assignQ()
        for node in t.traverse(self_before=True):
            assert node.Q is Q

    def test_assignQ_single_overwrite(self):
        """RangeNode assignQ should overwrite root Q if new Q passed"""
        t = self.t1
        Q = ['a']
        Q2 = ['b']
        t.Q = Q
        t.assignQ(Q2)
        for node in t.traverse(self_before=True):
            assert node.Q is Q2
            assert not node.Q is Q

    def test_assignQ_multiple(self):
        """RangeNode assignQ should propagate multiple Qs"""
        t = self.t1
        Q1 = ['a']
        Q2 = ['b']
        Q3 = ['c']
        t.makeIdIndex()
        t.IdIndex[7].Q = Q1
        t.IdIndex[5].Q = Q2
        t.assignQ(Q3) 
        result = [i.Q for i in t.traverse(self_after=True)]
        assert t.Q is Q3
        self.assertEqual(result, [Q3,Q2,Q2,Q2,Q3,Q1,Q1,Q1,Q3])

    def test_assignQ_multiple_overwrite(self):
        """RangeNode assignQ should allow overwrite"""
        t = self.t1
        Q1 = ['a']
        Q2 = ['b']
        Q3 = ['c']
        t.makeIdIndex()
        t.IdIndex[7].Q = Q1
        t.IdIndex[5].Q = Q2
        t.assignQ(Q3, overwrite=True)
        for i in t.traverse(self_after=True):
            assert i.Q is Q3

    def test_assignQ_special(self):
       """RangeNode assignQ should work with special Qs"""
       t = self.t1
       Q1 = 'a'
       Q2 = 'b'
       Q3 = 'c'
       t.makeIdIndex()
       special = {7:Q1, 1:Q2}
       #won't work if no Q at root
       self.assertRaises(ValueError, t.assignQ, special_qs=special)
       t.assignQ(Q3, special_qs=special)
       result = [i.Q for i in t.traverse(self_after=True)]
       self.assertEqual(result, ['c','b','c','c','c','a','a','a','c'])
        

    def test_assignP(self):
        """RangeNode assignP should work when Qs set."""
        t = self.t1
        for i in t.traverse(self_before=True):
            i.Length = random() * 0.5 #range 0 to 0.5
        t.Q = Rates.random(DnaPairs)
        t.assignQ()
        t.assignP()
        t.assignIds()
        for node in t.traverse(self_after=True):
            if node.Parent is not None:
                self.assertFloatEqual(average(1-diag(node.P._data), axis=0), \
                    node.Length)
       
    def test_assignLength(self):
        """RangeNode assignLength should set branch length"""
        t = self.t1
        t.assignLength(0.3)
        for i in t.traverse(self_before=True):
            self.assertEqual(i.Length, 0.3)
                
    def test_evolve(self):
        """RangeNode evolve should work on a starting vector"""
        t = self.t1
        t.Q = Rates.random(DnaPairs)
        t.assignQ()
        t.assignLength(0.1)
        t.assignP()
        start = array([1,0,2,1,0,0,2,1,2,0,1,2,1,0,2,0,0,3,0,2,1,0,3,1,0,2,0,0,0,0,0,1,1,1,1,1,2,2,2,2,2,3,3,3,3,3,3])
        t.evolve(start)
        for i in t.traverse():
            self.assertEqual(len(i.Sequence), len(start))
            self.assertNotEqual(i.Sequence, start)
        #WARNING: Doesn't test base freqs etc. at this point, but those aren't
        #really evolve()'s responsibity (tested as self.P.mutate(seq) once
        #P is set, which we've already demonstrated works.)

    def test_assignPs(self):
        """RangeNode assignPs should assign multiple scaled P matrices"""
        t = self.t1
        for i in t.traverse(self_before=True):
            i.Length = random() * 0.5 #range 0 to 0.5
        t.Q = Rates.random(DnaPairs)
        t.assignQ()
        t.assignPs([1, 0.5, 0.25])
        t.assignIds()
        for node in t.traverse(self_after=True):
            if node.Parent is not None:
                self.assertEqual(len(node.Ps), 3)
                self.assertFloatEqual(average(1-diag(node.Ps[0]._data), axis=0), \
                    node.Length)
                self.assertFloatEqual(average(1-diag(node.Ps[1]._data), axis=0), \
                    0.5*node.Length)
                self.assertFloatEqual(average(1-diag(node.Ps[2]._data), axis=0), \
                    0.25*node.Length)

    def test_evolveSeqs(self):
        """PhlyoNode evolveSeqs should evolve multiple sequences"""
        t = self.t1
        for i in t.traverse(self_before=True):
            i.Length = 0.5
        t.Q = Rates.random(DnaPairs)
        t.assignQ()
        t.assignPs([1, 1, 0.1])
        t.assignIds()
        orig_seqs = [array(i) for i in [randint(0,4,200), randint(0,4,200), \
            randint(0,4,200)]]
        t.evolveSeqs(orig_seqs)
        for node in t.traverse():   #only look at leaves
            if node.Parent is not None:
                self.assertEqual(len(node.Sequences), 3)
                for orig, new in zip(orig_seqs, node.Sequences):
                    self.assertEqual(len(orig), len(new))
                    self.assertNotEqual(orig, new)
                assert sum(orig_seqs[1]!=node.Sequences[1]) > \
                        sum(orig_seqs[2]!=node.Sequences[2])

 


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""tests for Trie and compressed Trie class."""

__author__ = "Jens Reeder"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Jens Reeder"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Jens Reeder"
__email__ = "jens.reeder@gmail.com"
__status__ = "Prototype"


from cogent.util.unit_test import TestCase, main
from cogent.util.trie import Trie, Compressed_Trie, build_prefix_map, build_trie, \
    _build_prefix_map

class TrieTests(TestCase):
    
    def setUp(self):
        self.data = dict({"0": "ab", "1":"abababa", "2":"abab",
                          "3":"baba", "4":"ababaa","5":"a", "6":"abababa",
                          "7":"bab", "8":"babba"})
    
    def test_init(self):
        """Trie init should create an empty trie."""
        
        t = Trie()
        self.assertEqual(t.root.labels, [])
        self.assertEqual(t.root.children, {})
        
    def test_insert_find(self):
        """An added key should be found by find."""

        data = self.data
        
        t = Trie()
        for (label, seq) in data.iteritems():
            t.insert(seq, label)
           
        for  (label, seq) in data.iteritems():
            self.assertEqual(label in t.find(seq), True)
        self.assertEqual(t.find("cacgchagc"), [])
        self.assertEqual(t.find("abababa"), ["1","6"])

    def test_insert_unique(self):
        """insert_unique should insert only unique words."""

        data = self.data
        
        t = Trie()
        for (label, seq) in data.iteritems():
            t._insert_unique(seq, label)
                       
        self.assertEqual(t.find("ab"), [])
        self.assertEqual(t.find("cacgchagc"), [])
        self.assertEqual(t.find("abababa"), ["1"])


    def test_build_prefix_map(self):
        """prefix_map should map prefix strings."""
        
        self.assertEqual(dict(_build_prefix_map(self.data.iteritems())),
                         {'1': ['0', '2', '5', '6'],
                          '8': [],
                          '3': ['7'],
                          '4': []})

class Compressed_Trie_Tests(TestCase):
    
    def setUp(self):
        self.data = dict({"0": "ab", "1":"abababa", "2":"abab",
                          "3":"baba", "4":"ababaa","5":"a", "6":"abababa",
                          "7":"bab", "8":"babba"})

        self.trie = build_trie(self.data.iteritems())
    
    def test_init(self):
        """Trie init should create an empty trie."""
        
        t = Compressed_Trie()
        self.assertEqual(t.root.labels, [])
        self.assertEqual(t.root.children, {})
        self.assertEqual(t.root.key, "")
        
    def test_non_zero(self):
        """__non_zero__ should cehck for any data in the trie."""
        t = Compressed_Trie()
        self.assertEqual(t.__nonzero__(), False)
        self.assertEqual(self.trie.__nonzero__(), True)

    def test_len(self):
        """__len__ should return the number of seqs in the trie."""
        
        self.assertEqual(len(self.trie), 9)
        t = Compressed_Trie()
        self.assertEqual(len(t), 0)

    def test_size(self):
        """size should return the number of nodes in the trie."""
        
        self.assertEqual(self.trie.size(), 10)

        #empty trie contins only root node
        t = Compressed_Trie()
        self.assertEqual(size(t), 1)

    def test_to_string(self):
        """_to_string should create a string representation."""

        string_rep = """
key 
{
\tkey a['5']
\t{
\t\tkey b['0']
\t\t{
\t\t\tkey ab['2']
\t\t\t{
\t\t\t\tkey a
\t\t\t\t{
\t\t\t\t\tkey a['4']
\t\t\t\t}
\t\t\t\t{
\t\t\t\t\tkey ba['1', '6']
\t\t\t\t}
\t\t\t}
\t\t}
\t}
}
{
\tkey bab['7']
\t{
\t\tkey a['3']
\t}
\t{
\t\tkey ba['8']
\t}
}
"""
        self.assertEqual(str(self.trie), string_rep)
        
    def test_insert_find(self):
        """An added key should be found by find."""

        data = self.data
        t = Compressed_Trie()
        for (label, seq) in data.iteritems():
            t.insert(seq, label)
    
        for  (label, seq) in data.iteritems():
            self.assertEqual(label in t.find(seq), True)
        self.assertEqual(t.find("abababa"), ["1","6"])
        self.assertEqual(t.find("cacgchagc"), [])


    def test_prefixMap(self):
        """prefix_map (Compressed_Trie) should map prefix strings."""
        
        self.assertEqual(self.trie.prefixMap(),
                         {'1': ['6', '2', '0', '5'],
                          '8': ['7'],
                          '3': [], 
                          '4': []})
    
    def test_init(self):
        """Trie init should create an empty trie."""
        
        t = Trie()
        self.assertEqual(t.root.labels, [])
        self.assertEqual(t.root.children, {})
        
    def test_insert_find(self):
        """An added key should be found by find."""

        data = self.data
        
        t = Trie()
        for (label, seq) in data.iteritems():
            t.insert(seq, label)
           
        for  (label, seq) in data.iteritems():
            self.assertEqual(label in t.find(seq), True)
        self.assertEqual(t.find("cacgchagc"), [])
        self.assertEqual(t.find("abababa"), ["1","6"])

    def test_insert_unique(self):
        """insert_unique should insert only unique words."""

        data = self.data
        
        t = Trie()
        for (label, seq) in data.iteritems():
            t._insert_unique(seq, label)
                       
        self.assertEqual(t.find("ab"), [])
        self.assertEqual(t.find("cacgchagc"), [])
        self.assertEqual(t.find("abababa"), ["1"])


    def test_build_prefix_map(self):
        """prefix_map should map prefix strings."""
        
        self.assertEqual(dict(_build_prefix_map(self.data.iteritems())),
                         {'1': ['0', '2', '5', '6'],
                          '8': [],
                          '3': ['7'],
                          '4': []})
   
    def test_build_trie(self):
        """Build_trie should build trie from seqs."""
        
        t = build_trie(self.data.iteritems(), Trie)
        self.assertTrue(isinstance(t, Trie))
        for (label, seq) in self.data.iteritems():
            self.assertContains(t.find(seq), label)

        self.assertEqual(t.find(""), [])
        self.assertEqual(t.find("ccc"), [])


class Compressed_Trie_Tests(TestCase):
    
    def setUp(self):
        self.data = dict({"0": "ab", "1":"abababa", "2":"abab",
                          "3":"baba", "4":"ababaa","5":"a", "6":"abababa",
                          "7":"bab", "8":"babba"})

        self.trie = build_trie(self.data.iteritems())
    
    def test_init(self):
        """Trie init should create an empty trie."""
        
        t = Compressed_Trie()
        self.assertEqual(t.root.labels, [])
        self.assertEqual(t.root.children, {})
        self.assertEqual(t.root.key, "")
        
    def test_non_zero(self):
        """__non_zero__ should cehck for any data in the trie."""
        t = Compressed_Trie()
        self.assertEqual(t.__nonzero__(), False)
        self.assertEqual(self.trie.__nonzero__(), True)

    def test_len(self):
        """__len__ should return the number of seqs in the trie."""
        
        self.assertEqual(len(self.trie), 9)

    def test_size(self):
        """size should return the number of nodes in the trie."""
        
        self.assertEqual(self.trie.size(), 10)

    def test_to_string(self):
        """_to_string should create a string representation."""

        string_rep = """
key 
{
\tkey a['5']
\t{
\t\tkey b['0']
\t\t{
\t\t\tkey ab['2']
\t\t\t{
\t\t\t\tkey a
\t\t\t\t{
\t\t\t\t\tkey a['4']
\t\t\t\t}
\t\t\t\t{
\t\t\t\t\tkey ba['1', '6']
\t\t\t\t}
\t\t\t}
\t\t}
\t}
}
{
\tkey bab['7']
\t{
\t\tkey a['3']
\t}
\t{
\t\tkey ba['8']
\t}
}
"""
        self.assertEqual(str(self.trie), string_rep)
        
    def test_insert_find(self):
        """An added key should be found by find."""

        data = self.data
        t = Compressed_Trie()
        for (label, seq) in data.iteritems():
            t.insert(seq, label)
    
        for  (label, seq) in data.iteritems():
            self.assertEqual(label in t.find(seq), True)
        self.assertEqual(t.find("abababa"), ["1","6"])
        self.assertEqual(t.find("cacgchagc"), [])


    def test_prefixMap(self):
        """prefix_map (Compressed_Trie) should map prefix strings."""
        
        self.assertEqual(self.trie.prefixMap(),
                         {'1': ['6', '2', '0', '5'],
                          '8': ['7'],
                          '3': [], 
                          '4': []})

    def test_build_trie(self):
        """Build_trie should build trie from seqs."""
        
        t = build_trie(self.data.iteritems())
        
        self.assertTrue(isinstance(t, Compressed_Trie))
        
        for (label, seq) in self.data.iteritems():
            self.assertContains(t.find(seq), label)

        self.assertEqual(t.find(""), [])
        self.assertEqual(t.find("ccc"), [])


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from cogent.util.unit_test import TestCase, main

from cogent.parse.fastq import MinimalFastqParser

__author__ = "Gavin Huttley"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Gavin Huttley"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"

data = {
"GAPC_0015:6:1:1259:10413#0/1":
        dict(seq='AACACCAAACTTCTCCACCACGTGAGCTACAAAAG',
             qual=r'````Y^T]`]c^cabcacc`^Lb^ccYT\T\Y\WF'),
"GAPC_0015:6:1:1283:11957#0/1":
        dict(seq='TATGTATATATAACATATACATATATACATACATA',
             qual=r']KZ[PY]_[YY^```ac^\\`bT``c`\aT``bbb'),
"GAPC_0015:6:1:1284:10484#0/1":
        dict(seq='TCAGTTTTCCTCGCCATATTTCACGTCCTAAAGCG',
             qual=r'UM_]]U_]Z_Y^\^^``Y]`^SZ]\Ybb`^_LbL_'),
"GAPC_0015:6:1:1287:17135#0/1":
        dict(seq='TGTGCCTATGGAAGCAGTTCTAGGATCCCCTAGAA',
             qual=r'^aacccL\ccc\c\cTKTS]KZ\]]I\[Wa^T`^K'),
"GAPC_0015:6:1:1293:3171#0/1":
        dict(seq="AAAGAAAGGAAGAAAAGAAAAAGAAACCCGAGTTA",
             qual=r"b`bbbU_[YYcadcda_LbaaabWbaacYcc`a^c"),
"GAPC_0015:6:1:1297:10729#0/1":
        dict(seq="TAATGCCAAAGAAATATTTCCAAACTACATGCTTA",
             qual=r"T\ccLbb``bacc]_cacccccLccc\ccTccYL^"),
"GAPC_0015:6:1:1299:5940#0/1":
        dict(seq="AATCAAGAAATGAAGATTTATGTATGTGAAGAATA",
             qual=r"dcddbcfffdfffd`dd`^`c`Oc`Ybb`^eecde"),
"GAPC_0015:6:1:1308:6996#0/1":
        dict(seq="TGGGACACATGTCCATGCTGTGGTTTTAACCGGCA",
             qual=r"a]`aLY`Y^^ccYa`^^TccK_X]\c\c`caTTTc"),
"GAPC_0015:6:1:1314:13295#0/1":
        dict(seq="AATATTGCTTTGTCTGAACGATAGTGCTCTTTGAT",
             qual=r"cLcc\\dddddaaYd`T```bLYT\`a```bZccc"),
"GAPC_0015:6:1:1317:3403#0/1":
        dict(seq="TTGTTTCCACTTGGTTGATTTCACCCCTGAGTTTG",
             qual=r"\\\ZTYTSaLbb``\_UZ_bbcc`cc^[ac\a\T\ ".strip())# had to add space
}
class ParseFastq(TestCase):
    def test_parse(self):
        """sequence and info objects should correctly match"""
        for label, seq, qual in MinimalFastqParser('data/fastq.txt'):
            self.assertTrue(label in data)
            self.assertEqual(seq, data[label]["seq"])
            self.assertEqual(qual, data[label]["qual"])
    

if __name__ == "__main__":
    main()




#!/usr/bin/env python

"""Application controller for Sfold application

Option -o sets output directory,default ../output/
If working directory is changed during the work flow option -o
need to be reset. -o is currently set to cwd
"""

from cogent.app.util import CommandLineApplication,\
    CommandLineAppResult, ResultPath
from cogent.app.parameters import Parameter, FlagParameter, ValuedParameter,\
    MixedParameter,Parameters 
from os import getcwd
from sys import stderr

__author__ = "Shandy Wikman"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__contributors__ = ["Shandy Wikman"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Shandy Wikman"
__email__ = "ens01svn@cs.umu.se"
__status__ = "Development"

#IMPORTANT!!!!!
#must be specified if run outside sfold dir, default is ../param/ 
param_dir = 'your_path/sfold-2.0/param/'


class Sfold(CommandLineApplication):
    """Application controller Sfold application"""

    cwd = getcwd()
    _parameters = {
        '-o':ValuedParameter(Prefix='-',Name='o',Delimiter=' ',Value=cwd),
        '-a':ValuedParameter(Prefix='-',Name='a',Delimiter=' '),
        '-f':ValuedParameter(Prefix='-',Name='f',Delimiter=' '),
        '-l':ValuedParameter(Prefix='-',Name='l',Delimiter=' '),
        '-m':ValuedParameter(Prefix='-',Name='m',Delimiter=' '),
        '-w':ValuedParameter(Prefix='-',Name='w',Delimiter=' '),
        '-p':ValuedParameter(Prefix='-',Name='p',Delimiter=' ',Value=param_dir)
        }
    _command = 'sfold.X86_64.LINUX'
    _input_handler = '_input_as_string'


    def _get_result_paths(self,data):
        """Return a dict of ResultPath objects representing all possible output
        """
        result = {}

        #list of output files
        output_list = ['10structure','10structure_2','Dharmacon_thermo','bp',
                       'bprob','cdf','fe','loopr','oligo','oligo_f','pdf',
                       'sample','sample_1000','sirna','sirna_f','sirna_s',
                       'smfe','sstrand','stability']

        for name in output_list:
            try:
                path = '%s%s.out' % (self.WorkingDir,name)
                f = open(path)
                f.close()
                result[name] = \
                    ResultPath(Path=path)
            except IOError:
                stderr.write('path: %s\n' % self.WorkingDir)
                stderr.write('File: %s.out not written to result dict\n' % name)
                pass

        return result





#!/usr/bin/env python

import random, numpy

from cogent.core.alignment import Alignment
from cogent.util.dict_array import DictArrayTemplate
from cogent.evolve.simulate import AlignmentEvolver, randomSequence
from cogent.util import parallel, table
from cogent.recalculation.definition import ParameterController

from cogent.util.warning import discontinued, deprecated

__author__ = "Peter Maxwell"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Gavin Huttley", "Andrew Butterfield", "Peter Maxwell",
                    "Matthew Wakefield", "Rob Knight", "Brett Easton"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"

# cogent.evolve.parameter_controller.LikelihoodParameterController tells the
# recalculation framework to use this subclass rather than the generic
# recalculation Calculator.  It adds methods which are useful for examining
# the parameter, psub, mprob and likelihood values after the optimisation is
# complete.

class LikelihoodFunction(ParameterController):
    def setpar(self, param_name, value, edge=None, **scope):
        deprecated('method', 'setpar','setParamRule', '1.6')
        return self.setParamRule(param_name, edge=edge, value=value, is_constant=True, **scope)
    
    def testfunction(self):
        deprecated('method', 'testfunction','getLogLikelihood', '1.6')
        return self.getLogLikelihood()
    
    def getLogLikelihood(self):
        return self.getFinalResult()
    
    def getPsubForEdge(self, name, **kw):
        """returns the substitution probability matrix for the named edge"""
        try:
            # For PartialyDiscretePsubsDefn
            array = self.getParamValue('dpsubs', edge=name, **kw)
        except KeyError:
            array = self.getParamValue('psubs', edge=name, **kw)
        return DictArrayTemplate(self._motifs, self._motifs).wrap(array)
    
    def getRateMatrixForEdge(self, name, **kw):
        """returns the rate matrix (Q) for the named edge
        
        Note: expm(Q) will give the same result as getPsubForEdge(name)"""
        try:
            array = self.getParamValue('Q', edge=name, **kw)
        except KeyError as err:
            if err[0] == 'Q' and name != 'Q':
                raise RuntimeError('rate matrix not known by this model')
            else:
                raise
        return DictArrayTemplate(self._motifs, self._motifs).wrap(array)
    
    def _getLikelihoodValuesSummedAcrossAnyBins(self, locus=None):
        if self.bin_names and len(self.bin_names) > 1:
            root_lhs = [self.getParamValue('lh', locus=locus, bin=bin) for
                bin in self.bin_names]
            bprobs = self.getParamValue('bprobs')
            root_lh = bprobs.dot(root_lhs)
        else:
            root_lh = self.getParamValue('lh', locus=locus)
        return root_lh
        
    def getFullLengthLikelihoods(self, locus=None):
        """Array of [site, motif] likelihoods from the root of the tree"""
        root_lh = self._getLikelihoodValuesSummedAcrossAnyBins(locus=locus)
        root_lht = self.getParamValue('root', locus=locus)
        return root_lht.getFullLengthLikelihoods(root_lh)
    
    def getGStatistic(self, return_table=False, locus=None):
        """Goodness-of-fit statistic derived from the unambiguous columns"""
        root_lh = self._getLikelihoodValuesSummedAcrossAnyBins(locus=locus)
        root_lht = self.getParamValue('root', locus=locus)
        return root_lht.calcGStatistic(root_lh, return_table)
    
    def reconstructAncestralSeqs(self, locus=None):
        """returns a dict of DictArray objects containing probabilities
        of each alphabet state for each node in the tree.
        
        Arguments:
            - locus: a named locus"""
        result = {}
        array_template = None
        for restricted_edge in self._tree.getEdgeVector():
            if restricted_edge.istip():
                continue
            try:
                r = []
                for motif in range(len(self._motifs)):
                    self.setParamRule('fixed_motif', value=motif,
                            edge=restricted_edge.Name, locus=locus,
                            is_constant=True)
                    likelihoods = self.getFullLengthLikelihoods(locus=locus)
                    r.append(likelihoods)
                    if array_template is None:
                        array_template = DictArrayTemplate(
                                likelihoods.shape[0], self._motifs)
            finally:
                self.setParamRule('fixed_motif', value=-1,
                        edge=restricted_edge.Name, locus=locus,
                        is_constant=True)
            # dict of site x motif arrays
            result[restricted_edge.Name] = array_template.wrap(
                    numpy.transpose(numpy.asarray(r)))
        return result
    
    def likelyAncestralSeqs(self, locus=None):
        """Returns the most likely reconstructed ancestral sequences as an
        alignment.
        
        Arguments:
            - locus: a named locus"""
        prob_array = self.reconstructAncestralSeqs(locus=locus)
        seqs = []
        for edge, probs in prob_array.items():
            seq = []
            for row in probs:
                by_p = [(p,state) for state, p in row.items()]
                seq.append(max(by_p)[1])
            seqs += [(edge, self.model.MolType.makeSequence("".join(seq)))]
        return Alignment(data = seqs, MolType = self.model.MolType)
    
    def getBinProbs(self, locus=None):
        hmm = self.getParamValue('bindex', locus=locus)
        lhs = [self.getParamValue('lh', locus=locus, bin=bin)
                for bin in self.bin_names]
        array = hmm.getPosteriorProbs(*lhs)
        return DictArrayTemplate(self.bin_names, array.shape[1]).wrap(array)
    
    def _valuesForDimension(self, dim):
        # in support of __str__
        if dim == 'edge':
            result = [e.Name for e in self._tree.getEdgeVector()]
        elif dim == 'bin':
            result = self.bin_names[:]
        elif dim == 'locus':
            result = self.locus_names[:]
        elif dim.startswith('motif'):
            result = self._mprob_motifs
        elif dim == 'position':
            result = self.posn_names[:]
        else:
            raise KeyError, dim
        return result
    
    def _valuesForDimensions(self, dims):
        # in support of __str__
        result = [[]]
        for dim in dims:
            new_result = []
            for r in result:
                for cat in self._valuesForDimension(dim):
                    new_result.append(r+[cat])
            result = new_result
        return result
    
    def __str__(self):
        if not self._name:
            title = 'Likelihood Function Table'
        else:
            title = self._name
        result = [title]
        result += self.getStatistics(with_motif_probs=True, with_titles=False)
        return '\n'.join(map(str, result))
    
    def getAnnotatedTree(self):
        d = self.getParamValueDict(['edge'])
        tree = self._tree.deepcopy()
        for edge in tree.getEdgeVector():
            if edge.Name == 'root':
                continue
            for par in d:
                edge.params[par] = d[par][edge.Name]
        return tree
    
    def getMotifProbs(self, edge=None, bin=None, locus=None):
        motif_probs_array = self.getParamValue(
                'mprobs', edge=edge, bin=bin, locus=locus)
        return DictArrayTemplate(self._mprob_motifs).wrap(motif_probs_array)
        #return dict(zip(self._motifs, motif_probs_array))
    
    def getBinPriorProbs(self, locus=None):
        bin_probs_array = self.getParamValue('bprobs', locus=locus)
        return DictArrayTemplate(self.bin_names).wrap(bin_probs_array)
    
    def getScaledLengths(self, predicate, bin=None, locus=None):
        """A dictionary of {scale:{edge:length}}"""
        if not hasattr(self._model, 'getScaledLengthsFromQ'):
            return {}
        
        def valueOf(param, **kw):
            return self.getParamValue(param, locus=locus, **kw)
        
        if bin is None:
            bin_names = self.bin_names
        else:
            bin_names = [bin]
        
        if len(bin_names) == 1:
            bprobs = [1.0]
        else:
            bprobs = valueOf('bprobs')
        
        mprobs = [valueOf('mprobs', bin=b) for b in bin_names]
        
        scaled_lengths = {}
        for edge in self._tree.getEdgeVector():
            if edge.isroot():
                continue
            Qs = [valueOf('Qd', bin=b, edge=edge.Name).Q for b in bin_names]
            length = valueOf('length', edge=edge.Name)
            scaled_lengths[edge.Name] = length * self._model.getScaleFromQs(
                    Qs, bprobs, mprobs, predicate)
        return scaled_lengths
    
    def getStatistics(self, with_motif_probs=True, with_titles=True):
        """returns the parameter values as tables/dict
        
        Arguments:
            - with_motif_probs: include the motif probability table
            - with_titles: include a title for each table based on it's
              dimension"""
        result = []
        group = {}
        param_names = self.getParamNames()
        
        mprob_name = [n for n in param_names if 'mprob' in n]
        if mprob_name:
            mprob_name = mprob_name[0]
        else:
            mprob_name = ''
        
        if not with_motif_probs:
            param_names.remove(mprob_name)
        
        for param in param_names:
            dims = tuple(self.getUsedDimensions(param))
            if dims not in group:
                group[dims] = []
            group[dims].append(param)
        table_order = group.keys()
        for table_dims in table_order:
            raw_table = self.getParamValueDict(
                dimensions=table_dims, params=group[table_dims])
            param_names = group[table_dims]
            param_names.sort()
            if table_dims == ('edge',):
                if 'length' in param_names:
                    param_names.remove('length')
                    param_names.insert(0, 'length')
                raw_table['parent'] = dict([(e.Name, e.Parent.Name)
                        for e in self._tree.getEdgeVector()
                        if not e.isroot()])
                param_names.insert(0, 'parent')
            list_table = []
            heading_names = list(table_dims) + param_names
            row_order = self._valuesForDimensions(table_dims)
            for scope in row_order:
                row = {}
                row_used = False
                for param in param_names:
                    d = raw_table[param]
                    try:
                        for part in scope:
                            d = d[part]
                    except KeyError:
                            d = 'NA'
                    else:
                        row_used = True
                    row[param] = d
                if row_used:
                    row.update(dict(zip(table_dims, scope)))
                    row = [row[k] for k in heading_names]
                    list_table.append(row)
            if table_dims:
                title = ['', '%s params' % ' '.join(table_dims)][with_titles]
                row_ids = True
            else:
                row_ids = False
                title = ['', 'global params'][with_titles]
            result.append(table.Table(
                        heading_names, list_table,
                        max_width = 80, row_ids = row_ids,
                        title=title, **self._format))
        return result
    
    def getStatisticsAsDict(self, with_parent_names=True,
                with_edge_names=False):
        """Returns a dictionary containing the statistics for each edge of the
        tree, and any other information provided by the substitution model. The
        dictionary is keyed at the top-level by parameter name, and then by
        edge.name.
        
        Arguments:
            - with_edge_names: if True, an ordered list of edge names is
              included under the top-level key 'edge.names'. Default is
              False.
        """
        
        discontinued('method', "'getStatisticsAsDict' "
                "use 'getParamValueDict(['edge'])' is nearly equivalent", 
                '1.6')
        
        stats_dict = self.getParamValueDict(['edge'])
        
        if hasattr(self.model, 'scale_masks'):
            for predicate in self.model.scale_masks:
                stats_dict[predicate] = self.getScaledLengths(predicate)
        
        edge_vector = [e for e in self._tree.getEdgeVector() if not e.isroot()]
        
        # do the edge names
        if with_parent_names:
            parents = {}
            for edge in edge_vector:
                if edge.Parent.isroot():
                    parents[edge.Name] = "root"
                else:
                    parents[edge.Name] = str(edge.Parent.Name)
            stats_dict["edge.parent"] = parents
        
        if with_edge_names:
            stats_dict['edge.name'] = (
                    [e.Name for e in edge_vector if e.istip()] +
                    [e.Name for e in edge_vector if not e.istip()])
        
        return stats_dict
    
    # For tests.  Compat with old LF interface
    def setName(self, name):
        self._name = name
    
    def getName(self):
        return self._name or 'unnamed'
    
    def setTablesFormat(self, space=4, digits=4):
        """sets display properties for statistics tables. This affects results
        of str(lf) too."""
        space = [space, 4][type(space)!=int]
        digits = [digits, 4][type(digits)!=int]
        self._format = dict(space=space, digits=digits)
    
    def getMotifProbsByNode(self, edges=None, bin=None, locus=None):
        kw = dict(bin=bin, locus=locus)
        mprobs = self.getParamValue('mprobs', **kw)
        mprobs = self._model.calcWordProbs(mprobs)
        result = self._nodeMotifProbs(self._tree, mprobs, kw)
        if edges is None:
            edges = [name for (name, m) in result]
        result = dict(result)
        values = [result[name] for name in edges]
        return DictArrayTemplate(edges, self._mprob_motifs).wrap(values)
        
    def _nodeMotifProbs(self, tree, mprobs, kw):
        result = [(tree.Name, mprobs)]
        for child in tree.Children:
            psub = self.getPsubForEdge(child.Name, **kw)
            child_mprobs = numpy.dot(mprobs, psub)
            result.extend(self._nodeMotifProbs(child, child_mprobs, kw))
        return result
        
    def simulateAlignment(self, sequence_length=None, random_series=None,
            exclude_internal=True, locus=None, seed=None, root_sequence=None):
        """
        Returns an alignment of simulated sequences with key's corresponding to
        names from the current attached alignment.
        
        Arguments:
            - sequence_length: the legnth of the alignment to be simulated,
              default is the length of the attached alignment.
            - random_series: a random number generator.
            - exclude_internal: if True, only sequences for tips are returned.
            - root_sequence: a sequence from which all others evolve.
        """
        
        if sequence_length is None:
            lht = self.getParamValue('lht', locus=locus)
            sequence_length = len(lht.index)
            leaves = self.getParamValue('leaf_likelihoods', locus=locus)
            orig_ambig = {} #alignment.getPerSequenceAmbiguousPositions()
            for (seq_name, leaf) in leaves.items():
                orig_ambig[seq_name] = leaf.getAmbiguousPositions()
        else:
            orig_ambig = {}
        
        if random_series is None:
            random_series = random.Random()
            random_series.seed(seed)
            parallel.sync_random(random_series)
        
        def psub_for(edge, bin):
            return self.getPsubForEdge(edge, bin=bin, locus=locus)
        
        if len(self.bin_names) > 1:
            hmm = self.getParamValue('bdist', locus=locus)
            site_bins = hmm.emit(sequence_length, random_series)
        else:
            site_bins = numpy.zeros([sequence_length], int)
        
        evolver = AlignmentEvolver(random_series, orig_ambig, exclude_internal,
                self.bin_names, site_bins, psub_for, self._motifs)
        
        if root_sequence is not None: # we convert to a vector of motifs
            if isinstance(root_sequence, str):
                root_sequence = self._model.MolType.makeSequence(root_sequence)
            motif_len = self._model.getAlphabet().getMotifLen()
            root_sequence = root_sequence.getInMotifSize(motif_len)
        else:
            mprobs = self.getParamValue('mprobs', locus=locus, edge='root')
            mprobs = self._model.calcWordProbs(mprobs)
            mprobs = dict((m, p) for (m,p) in zip(self._motifs, mprobs))
            root_sequence = randomSequence(
                random_series, mprobs, sequence_length)
        
        simulated_sequences = evolver(self._tree, root_sequence)
        
        return Alignment(
                data = simulated_sequences,
                MolType = self._model.MolType)
        
    



#!/usr/bin/env python

__all__ = ["best_likelihood", "bootstrap", "likelihood_calculation",
           "likelihood_function", "likelihood_tree", "models",
           "pairwise_distance", "parameter_controller", "predicate",
           "simulate", "substitution_calculation", "substitution_model",
           "coevolution"]

__author__ = ""
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Gavin Huttley", "Peter Maxwell", "Andrew Butterfield",
                    "Rob Knight", "Matthrew Wakefield", "Brett Easton", 
                    "Edward Lang","Greg Caporaso"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"



#!/usr/bin/env python

__all__ = [ 'dendrogram', 'dotplot', 'linear', 'colors', 'TrackDefn',
            'Display', 'DisplayPolicy', 'Area', 'Arrow', 'BluntArrow', 
            'Box', 'Diamond'] + [
            'arrow_rates', 'dinuc', 'fancy_arrow', 'codon_usage','util']

__copyright__ = "Copyright 2007-2012, The Cogent Project"
__contributors__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight", 
                    "Zongzhi Liu", "Matthew Wakefield", "Stephanie Wilson"]
__license__ = "GPL"
__version__ = "1.5.3"
__status__ = "Production"

from cogent.draw.linear import (colors, TrackDefn, Display, DisplayPolicy, 
        Area, Arrow, BluntArrow, Box, Diamond)

try:
    import cogent.draw.matplotlib
except ImportError:
    pass
else:
    import warnings
    warnings.warn("You still have a cogent/draw/matplotlib subpackage" +
        " present, that will cause problems for matplotlib imports")



#!/usr/bin/env python
__all__ = ['test_methods', 'test_util']

__author__ = ""
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Sandra Smit", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Sandra Smit"
__email__ = "sandra.smit@colorado.edu"
__status__ = "Development"



#!/usr/bin/env python
"""Provides an application controller for the commandline version of
mothur Version 1.6.0
"""


from __future__ import with_statement
from operator import attrgetter
from os import path, getcwd, mkdir, remove, listdir, rmdir
import re
from shutil import copyfile
from subprocess import Popen
from tempfile import mkdtemp, NamedTemporaryFile
from cogent.app.parameters import ValuedParameter
from cogent.app.util import CommandLineApplication, ResultPath, \
    CommandLineAppResult, ApplicationError
from cogent.parse.mothur import parse_otu_list


__author__ = "Kyle Bittinger"
__copyright__ = "Copyright 2007-2012, The Cogent Project"
__credits__ = ["Kyle Bittinger"]
__license__ = "GPL"
__version__ = "1.5.3"
__maintainer__ = "Kyle Bittinger"
__email__ = "kylebittinger@gmail.com"
__status__ = "Prototype"


class Mothur(CommandLineApplication):
    """Mothur application controller
    """
    _options = {
        # Clustering algorithm.  Choices are furthest, nearest, and
        # average
        'method': ValuedParameter(
            Name='method', Value='furthest', Delimiter='=', Prefix=''),
        # Cutoff distance for the distance matrix
        'cutoff': ValuedParameter(
            Name='cutoff', Value=None, Delimiter='=', Prefix=''),
        # Minimum pairwise distance to consider for clustering
        'precision': ValuedParameter(
            Name='precision', Value=None, Delimiter='=', Prefix=''),
        }
    _parameters = {}
    _parameters.update(_options)
    _input_handler = '_input_as_multiline_string'
    _command = 'mothur'
    
    def __init__(self, params=None, InputHandler=None, SuppressStderr=None,
                 SuppressStdout=None, WorkingDir=None, TmpDir='/tmp',
                 TmpNameLen=20, HALT_EXEC=False):
        """Initialize a Mothur application controller
        
            params: a dictionary mapping the Parameter id or synonym to its
                value (or None for FlagParameters or MixedParameters in flag
                mode) for Parameters that should be turned on
            InputHandler: this is the method to be run on data when it is
                passed into call. This should be a string containing the
                method name. The default is _input_as_string which casts data
                to a string before appending it to the command line argument
            SuppressStderr: if set to True, will route standard error to
                /dev/null, False by default
            SuppressStdout: if set to True, will route standard out to
                /dev/null, False by default
            WorkingDir: the directory where you want the application to run,
                default is the current working directory, but is useful to 
                change in cases where the program being run creates output
                to its current working directory and you either don't want
                it to end up where you are running the program, or the user 
                running the script doesn't have write access to the current 
                working directory
                WARNING: WorkingDir MUST be an absolute path!
            TmpDir: the directory where temp files will be created, /tmp
                by default 
            TmpNameLen: the length of the temp file name
            HALT_EXEC: if True, raises exception w/ command output just
                before execution, doesn't clean up temp files. Default False.
        """
        super(Mothur, self).__init__(
            params=params, InputHandler=InputHandler, 
            SuppressStderr=SuppressStderr, SuppressStdout=SuppressStdout,
            WorkingDir='', TmpDir='', TmpNameLen=TmpNameLen, 
            HALT_EXEC=HALT_EXEC)
        # Prevent self.WorkingDir from being explicitly cast as a
        # FilePath object.  This behavior does not seem necessary in
        # the parent's __init__() method, since the casting is
        # repeated in _set_WorkingDir().
        if WorkingDir is not None:
            working_dir = WorkingDir
        else:
            working_dir = self._working_dir or getcwd()
        self.WorkingDir = working_dir
        self.TmpDir = TmpDir

    @staticmethod
    def getHelp():
        """Returns link to online manual"""
        help = (
            'See manual, available on the MOTHUR wiki:\n'
            'http://schloss.micro.umass.edu/mothur/'
            )
        return help

    def __call__(self, data=None, remove_tmp=True):
        """Run the application with the specified kwargs on data
        
            data: anything that can be cast into a string or written out to
                a file. Usually either a list of things or a single string or 
                number. input_handler will be called on this data before it 
                is passed as part of the command-line argument, so by creating
                your own input handlers you can customize what kind of data
                you want your application to accept

            remove_tmp: if True, removes tmp files
        """
        # Process the input data.  Input filepath is stored in
        # self._input_filename
        getattr(self, self.InputHandler)(data)

        if self.SuppressStdout:
            outfile = None
        else:
            outfile = open(self.getTmpFilename(self.TmpDir), 'w')
        if self.SuppressStderr:
            errfile = None
        else:
            errfile = open(self.getTmpFilename(self.TmpDir), 'w')

        args = [self._command, self._compile_mothur_script()]
        process = Popen(
            args, stdout=outfile, stderr=errfile, cwd=self.WorkingDir)
        exit_status = process.wait()
        if not self._accept_exit_status(exit_status):
            raise ApplicationError(
                'Unacceptable application exit status: %s, command: %s' % \
                    (exit_status, args))

        if outfile is not None:
            outfile.seek(0)
        if errfile is not None:
            errfile.seek(0)
        result = CommandLineAppResult(
            outfile, errfile, exit_status, result_paths=self._get_result_paths())

        # Clean up the input file if one was created
        if remove_tmp:
            if self._input_filename:
                remove(self._input_filename)
                self._input_filename = None

        return result

    def _accept_exit_status(self, status):
        return int(status) == 0

    def _compile_mothur_script(self):
        """Returns a Mothur batch script as a string"""
        def format_opts(*opts):
            """Formats a series of options for a Mothur script"""
            return ', '.join(filter(None, map(str, opts)))
        vars = {
            'in': self._input_filename,
            'unique': self._derive_unique_path(),
            'dist': self._derive_dist_path(),
            'names': self._derive_names_path(),
            'cluster_opts' : format_opts(
                self.Parameters['method'],
                self.Parameters['cutoff'],
                self.Parameters['precision'],
                ),
            }
        script = (
            '#'
            'unique.seqs(fasta=%(in)s); '
            'dist.seqs(fasta=%(unique)s); '
            'read.dist(column=%(dist)s, name=%(names)s); '
            'cluster(%(cluster_opts)s)' % vars
            )
        return script

    def _get_result_paths(self):
        paths = {
            'distance matrix': self._derive_dist_path(),
            'otu list': self._derive_list_path(),
            'rank abundance': self._derive_rank_abundance_path(),
            'species abundance': self._derive_species_abundance_path(),
            'unique names': self._derive_names_path(),
            'unique seqs': self._derive_unique_path(),
            'log': self._derive_log_path(),
            }
        return dict([(k, ResultPath(v)) for (k,v) in paths.items()])

    # Methods to derive/guess output pathnames produced by MOTHUR.
    # TODO: test for input files that do not have a filetype extension

    def _derive_log_path(self):
        """Guess logfile path produced by Mothur

        This method checks the working directory for log files
        generated by Mothur.  It will raise an ApplicationError if no
        log file can be found.

        Mothur generates log files named in a nondeterministic way,
        using the current time.  We return the log file with the most
        recent time, although this may lead to incorrect log file
        detection if you are running many instances of mothur
        simultaneously.
        """
        filenames = listdir(self.WorkingDir)
        lognames = [x for x in filenames if re.match("^mothur\.\d+\.logfile$", x)]
        if not lognames:
            raise ApplicationError(
                'No log file detected in directory %s. Contents: \n\t%s' % (
                    input_dir, '\n\t'.join(possible_logfiles)))
        most_recent_logname = sorted(lognames, reverse=True)[0]
        return path.join(self.WorkingDir, most_recent_logname)

    def _derive_unique_path(self):
        """Guess unique sequences path produced by Mothur"""
        base, ext = path.splitext(self._input_filename)
        return '%s.unique%s' % (base, ext)

    def _derive_dist_path(self):
        """Guess distance matrix path produced by Mothur"""
        base, ext = path.splitext(self._input_filename)
        return '%s.unique.dist' % base

    def _derive_names_path(self):
        """Guess unique names file path produced by Mothur"""
        base, ext = path.splitext(self._input_filename)
        return '%s.names' % base

    def __get_method_abbrev(self):
        """Abbreviated form of clustering method parameter.

        Used to guess output filenames for MOTHUR.
        """
        abbrevs = {
            'furthest': 'fn',
            'nearest':  'nn',
            'average':  'an',
        }
        if self.Parameters['method'].isOn():
            method = self.Parameters['method'].Value
        else:
            method = self.Parameters['method'].Default
        return abbrevs[method]

    def _derive_list_path(self):
        """Guess otu list file path produced by Mothur"""
        base, ext = path.splitext(self._input_filename)
        return '%s.unique.%s.list' % (base, self.__get_method_abbrev())
        
    def _derive_rank_abundance_path(self):
        """Guess rank abundance file path produced by Mothur"""
        base, ext = path.splitext(self._input_filename)
        return '%s.unique.%s.rabund' % (base, self.__get_method_abbrev())
        
    def _derive_species_abundance_path(self):
        """Guess species abundance file path produced by Mothur"""
        base, ext = path.splitext(self._input_filename)
        return '%s.unique.%s.sabund' % (base, self.__get_method_abbrev())

    def getTmpFilename(self, tmp_dir='/tmp', prefix='tmp', suffix='.txt'):
        """Returns a temporary filename

        Similar interface to tempfile.mktmp()
        """
        # Override to change default constructor to str(). FilePath
        # objects muck up the Mothur script.
        return super(Mothur, self).getTmpFilename(
            tmp_dir=tmp_dir, prefix=prefix, suffix=suffix, 
            result_constructor=str)

    # Temporary input file needs to be in the working directory, so we
    # override all input handlers.

    def _input_as_multiline_string(self, data):
        """Write multiline string to temp file, return filename

        data: a multiline string to be written to a file.
        """
        self._input_filename = self.getTmpFilename(suffix='.fasta')
        with open(self._input_filename, 'w') as f:
            f.write(data)
        return self._input_filename

    def _input_as_lines(self, data):
        """Write sequence of lines to temp file, return filename
        
        data: a sequence to be written to a file, each element of the
            sequence will compose a line in the file

        * Note: '\n' will be stripped off the end of each sequence
            element before writing to a file in order to avoid
            multiple new lines accidentally be written to a file
        """
        self._input_filename = self.getTmpFilename(suffix='.fasta')
        with open(self._input_filename, 'w') as f:
            # Use lazy iteration instead of list comprehension to
            # prevent reading entire file into memory
            for line in data:
                f.write(str(line).strip('\n'))
                f.write('\n')
        return self._input_filename

    def _input_as_path(self, data):
        """Copys the provided file to WorkingDir and returns the new filename

        data: path or filename
        """
        self._input_filename = self.getTmpFilename(suffix='.fasta')
        copyfile(data, self._input_filename)
        return self._input_filename

    def _input_as_paths(self, data):
        raise NotImplementedError('Not applicable for MOTHUR controller.')

    def _input_as_string(self, data):
        raise NotImplementedError('Not applicable for MOTHUR controller.')

    # FilePath objects muck up the Mothur script, so we override the
    # property methods for self.WorkingDir

    def _get_WorkingDir(self):
        """Gets the working directory"""
        return self._curr_working_dir

    def _set_WorkingDir(self, path):
        """Sets the working directory
        """
        self._curr_working_dir = path
        try:
            mkdir(self.WorkingDir)
        except OSError:
            # Directory already exists
            pass

    WorkingDir = property(_get_WorkingDir, _set_WorkingDir)


def mothur_from_file(file):
    app = Mothur(InputHandler='_input_as_lines')
    result = app(file)
    # Force evaluation, so we can safely clean up files
    otus = list(parse_otu_list(result['otu list']))
    result.cleanUp()
    return otus


class MothurClassifySeqs(Mothur):
    _options = {
        'reference': ValuedParameter(
            Name='reference', Value=None, Delimiter='=', Prefix=''),
        'taxonomy': ValuedParameter(
            Name='taxonomy', Value=None, Delimiter='=', Prefix=''),
        'cutoff': ValuedParameter(
            Name='cutoff', Value=None, Delimiter='=', Prefix=''),
        'iters': ValuedParameter(
            Name='iters', Value=None, Delimiter='=', Prefix=''),
        'ksize': ValuedParameter(
            Name='ksize', Value=None, Delimiter='=', Prefix=''),
        }
    _parameters = {}
    _parameters.update(_options)

    def _format_function_arguments(self, opts):
        """Format a series of function arguments in a Mothur script."""
        params = [self.Parameters[x] for x in opts]
        return ', '.join(filter(None, map(str, params)))

    def _compile_mothur_script(self):
        """Returns a Mothur batch script as a string"""
        fasta = self._input_filename

        required_params = ["reference", "taxonomy"]
        for p in required_params:
            if self.Parameters[p].Value is None:
                raise ValueError("Must provide value for parameter %s" % p)
        optional_params = ["ksize", "cutoff", "iters"]
        args = self._format_function_arguments(
            required_params + optional_params)
        script = '#classify.seqs(fasta=%s, %s)' % (fasta, args)
        return script

    def _get_result_paths(self):
        input_base, ext = path.splitext(path.basename(self._input_filename))
        result_by_suffix = {
            ".summary": "summary",
            ".taxonomy": "assignments",
            ".accnos": "accnos",
            }
        
        paths = {'log': self._derive_log_path()}
        input_dir = path.dirname(self._input_filename)
        for fn in listdir(input_dir):
            if fn.startswith(input_base):
                for suffix, result_key in result_by_suffix.items():
                    if fn.endswith(suffix):
                        paths[result_key] = path.join(input_dir, fn)
        return dict([(k, ResultPath(v)) for (k,v) in paths.items()])


def parse_mothur_assignments(lines):
    for line in lines:
        line = line.strip()
        if not line:
            continue
        seq_id, _, assignment = line.partition("\t")
        toks = assignment.rstrip(";").split(";")
        lineage = []
        conf = None
        for tok in toks:
            matchobj = re.match("(.+)\((\d+)\)$", tok)
            if matchobj:
                lineage.append(matchobj.group(1))
                pct_conf = int(matchobj.group(2))
                conf = pct_conf / 100.0
        yield seq_id, lineage, conf


def mothur_classify_file(
    query_file, ref_fp, tax_fp, cutoff=None, iters=None, ksize=None,
    output_fp=None):
    """Classify a set of sequences using Mothur's naive bayes method

    Dashes are used in Mothur to provide multiple filenames.  A
    filepath with a dash typically breaks an otherwise valid command
    in Mothur.  This wrapper script makes a copy of both files, ref_fp
    and tax_fp, to ensure that the path has no dashes.

    For convenience, we also ensure that each taxon list in the
    id-to-taxonomy file ends with a semicolon.
    """
    tmp_ref_file = NamedTemporaryFile(suffix=".ref.fa")
    for line in open(ref_fp):
        tmp_ref_file.write(line)
    tmp_ref_file.seek(0)

    tmp_tax_file = NamedTemporaryFile(suffix=".tax.txt")
    for line in open(tax_fp):
        line = line.rstrip()
        if not line.endswith(";"):
            line = line + ";"
        tmp_tax_file.write(line)
        tmp_tax_file.write("\n")
    tmp_tax_file.seek(0)

    params = {"reference": tmp_ref_file.name, "taxonomy": tmp_tax_file.name}
    if cutoff is not None:
        params["cutoff"] = cutoff
    if ksize is not None:
        params["ksize"] = ksize
    if iters is not None:
        params["iters"] = iters

    app = MothurClassifySeqs(params, InputHandler='_input_as_lines')
    result = app(query_file)

    # Force evaluation so we can safely clean up files
    assignments = list(parse_mothur_assignments(result['assignments']))
    result.cleanUp()

    if output_fp is not None:
        f = open(output_fp, "w")
        for query_id, taxa, conf in assignments:
            taxa_str = ";".join(taxa)
            f.write("%s\t%s\t%.2f\n" % (query_id, taxa_str, conf))
        f.close()
        return None
    return dict((a, (b, c)) for a, b, c in assignments)
